# Copyright (c) 2018 Cisco and/or its affiliates.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
author:
 - Ansible Networking Team (@ansible-network)
name: restconf
short_description: HttpApi Plugin for devices supporting Restconf API
description:
- This HttpApi plugin provides methods to connect to Restconf API endpoints.
version_added: 1.0.0
options:
  root_path:
    type: str
    description:
    - Specifies the location of the Restconf root.
    default: /restconf
    vars:
    - name: ansible_httpapi_restconf_root
"""

import json

from ansible.module_utils._text import to_text
from ansible.module_utils.connection import ConnectionError
from ansible.module_utils.six.moves.urllib.error import HTTPError

from ansible_collections.ansible.netcommon.plugins.plugin_utils.httpapi_base import HttpApiBase


CONTENT_TYPE = "application/yang-data+json"


class HttpApi(HttpApiBase):
    def send_request(self, data, **message_kwargs):
        if data:
            data = json.dumps(data)

        path = "/".join(
            [
                self.get_option("root_path").rstrip("/"),
                message_kwargs.get("path", "").lstrip("/"),
            ]
        )

        headers = {
            "Content-Type": message_kwargs.get("content_type") or CONTENT_TYPE,
            "Accept": message_kwargs.get("accept") or CONTENT_TYPE,
        }
        response, response_data = self.connection.send(
            path, data, headers=headers, method=message_kwargs.get("method")
        )

        return handle_response(response, response_data)


def handle_response(response, response_data):
    try:
        response_data = json.loads(response_data.read())
    except ValueError:
        response_data.seek(0)
        response_data = response_data.read()

    if isinstance(response, HTTPError):
        if response_data:
            if "errors" in response_data:
                errors = response_data["errors"]["error"]
                error_text = "\n".join((error["error-message"] for error in errors))
            else:
                error_text = response_data

            raise ConnectionError(error_text, code=response.code)
        raise ConnectionError(to_text(response), code=response.code)

    return response_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
options:
    dnac_host:
        description:
          - The Cisco DNA Center hostname.
        type: str
        required: true
    dnac_port:
        description:
          - The Cisco DNA Center port.
        type: str
        default: '443'
    dnac_username:
        description:
          - The Cisco DNA Center username to authenticate.
        type: str
        default: admin
        aliases: [ user ]
    dnac_password:
        description:
          - The Cisco DNA Center password to authenticate.
        type: str
    dnac_verify:
        description:
          - Flag to enable or disable SSL certificate verification.
        type: bool
        default: true
    dnac_version:
        description:
          - Informs the SDK which version of Cisco DNA Center to use.
        type: str
        default: 2.2.3.3
    dnac_debug:
        description:
          - Flag for Cisco DNA Center SDK to enable debugging.
        type: bool
        default: false
    dnac_log:
        description:
          - Flag to enable/disable playbook execution logging.
          - When true and dnac_log_file_path is provided,
            - Create the log file at the execution location with the specified name.
          - When true and dnac_log_file_path is not provided,
            - Create the log file at the execution location with the name 'dnac.log'.
          - When false,
            - Logging is disabled.
          - If the log file doesn't exist,
            - It is created in append or write mode based on the "dnac_log_append" flag.
          - If the log file exists,
            - It is overwritten or appended based on the "dnac_log_append" flag.
        type: bool
        default: false
    dnac_log_level:
        description:
          - Sets the threshold for log level. Messages with a level equal to or higher than
            this will be logged. Levels are listed in order of severity [CRITICAL, ERROR, WARNING, INFO, DEBUG].
          - CRITICAL indicates serious errors halting the program. Displays only CRITICAL messages.
          - ERROR indicates problems preventing a function. Displays ERROR and CRITICAL messages.
          - WARNING indicates potential future issues. Displays WARNING, ERROR, CRITICAL messages.
          - INFO tracks normal operation. Displays INFO, WARNING, ERROR, CRITICAL messages.
          - DEBUG provides detailed diagnostic info. Displays all log messages.
        type: str
        default: WARNING
    dnac_log_file_path:
        description:
        - Governs logging. Logs are recorded if dnac_log is True.
        - If path is not specified,
          - When 'dnac_log_append' is True, 'dnac.log' is generated in the
            current Ansible directory; logs are appended.
          - When 'dnac_log_append' is False, 'dnac.log' is generated; logs
            are overwritten.
        - If path is specified,
          - When 'dnac_log_append' is True, the file opens in append mode.
          - When 'dnac_log_append' is False, the file opens in write (w) mode.
          - In shared file scenarios, without append mode, content is
            overwritten after each module execution.
          - For a shared log file, set append to False for the 1st module
            (to overwrite); for subsequent modules, set append to True.
        type: str
        default: dnac.log
    dnac_log_append:
        description: Determines the mode of the file. Set to True for 'append' mode. Set to False for 'write' mode.
        type: bool
        default: True
    dnac_api_task_timeout:
      description:  Defines the timeout in seconds for API calls to retrieve task details. If the task details
          are not received within this period, the process will end, and a timeout notification will be logged.
      type: int
      default: 1200
    dnac_task_poll_interval:
      description: Specifies the interval in seconds between successive calls to the API to retrieve task details.
      type: int
      default: 2
    validate_response_schema:
        description:
          - Flag for Cisco DNA Center SDK to enable the validation of request bodies against a JSON schema.
        type: bool
        default: true
notes:
    - "Does not support C(check_mode)"
    - "The plugin runs on the control node and does not use any ansible connection plugins, but instead the embedded connection manager from Cisco DNAC SDK"
    - "The parameters starting with dnac_ are used by the Cisco DNAC Python SDK to establish the connection"
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: clients_trend_analytics_id_v1
short_description: Resource module for Clients Trend Analytics Id V1
description:
- Manage operation create of the resource Clients Trend Analytics Id V1.
- >
   Retrieves the time series information of a specific client by applying complex filters, aggregate functions, and
   grouping. The data will be grouped based on the specified trend time interval. For detailed information about the
   usage of the API, please refer to the Open API specification document - https //github.com/cisco-en-
   programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-clients1-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Clients Trend Analytics Id's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Clients Trend Analytics Id's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: int
    type: list
  groupBy:
    description: Group By.
    elements: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  id:
    description: Id path parameter. Id is the client mac address. It can be specified
      in one of the notational conventions 01 23 45 67 89 AB or 01-23-45-67-89-AB or
      0123.4567.89AB and is case insensitive.
    type: str
  page:
    description: Clients Trend Analytics Id's page.
    suboptions:
      cursor:
        description: Cursor.
        type: str
      limit:
        description: Limit.
        type: int
      timeSortOrder:
        description: Time Sort Order.
        type: str
    type: dict
  startTime:
    description: Start Time.
    type: int
  trendInterval:
    description: Trend Interval.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients RetrievesSpecificClientInformationOverASpecifiedPeriodOfTimeV1
  description: Complete reference of the RetrievesSpecificClientInformationOverASpecifiedPeriodOfTimeV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-specific-client-information-over-a-specified-period-of-time
notes:
  - SDK Method used are
    clients.Clients.retrieves_specific_client_information_over_a_specified_period_of_time_v1,

  - Paths used are
    post /dna/data/api/v1/clients/{id}/trendAnalytics,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.clients_trend_analytics_id_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - key: string
      operator: string
      value: 0
    groupBy:
    - string
    headers: '{{my_headers | from_json}}'
    id: string
    page:
      cursor: string
      limit: 0
      timeSortOrder: string
    startTime: 0
    trendInterval: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "timestamp": 0,
          "groups": [
            {
              "id": "string",
              "attributes": [
                {
                  "name": "string",
                  "value": 0
                }
              ],
              "aggregateAttributes": [
                {
                  "name": "string",
                  "function": "string",
                  "value": 0
                }
              ]
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "cursor": "string",
        "count": 0,
        "timeSortOrder": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: assurance_events_v1_info
short_description: Information module for Assurance Events V1
description:
- Get all Assurance Events V1.
- Get Assurance Events V1 by id.
- >
   API to fetch the details of an assurance event using event `id`. For detailed information about the usage of the
   API, please refer to the Open API specification document - https //github.com/cisco-en-programmability/catalyst-
   center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-AssuranceEvents-1.0.0-resolved.yaml.
- >
   Returns the list of events discovered by Catalyst Center, determined by the complex filters. Please refer to the
   'API Support Documentation' section to understand which fields are supported. For detailed information about the
   usage of the API, please refer to the Open API specification document - https //github.com/cisco-en-
   programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   AssuranceEvents-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceFamily:
    description:
    - >
      DeviceFamily query parameter. Device family. Please note that multiple families across network device type
      and client type is not allowed. For example, choosing `Routers` along with `Wireless Client` or `Unified AP`
      is not supported. Examples `deviceFamily=Switches and Hubs` (single deviceFamily requested)
      `deviceFamily=Switches and Hubs&deviceFamily=Routers` (multiple deviceFamily requested).
    type: str
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time minus 24 hours.
    type: float
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive. If `endTime` is not provided, API will
      default to current time.
    type: float
  messageType:
    description:
    - >
      MessageType query parameter. Message type for the event. Examples `messageType=Syslog` (single messageType
      requested) `messageType=Trap&messageType=Syslog` (multiple messageType requested).
    type: str
  severity:
    description:
    - >
      Severity query parameter. Severity of the event between 0 and 6. This is applicable only for events related
      to network devices (other than AP) and `Wired Client` events. | Value | Severity | | ----- | ----------- | |
      0 | Emergency | | 1 | Alert | | 2 | Critical | | 3 | Error | | 4 | Warning | | 5 | Notice | | 6 | Info |
      Examples `severity=0` (single severity requested) `severity=0&severity=1` (multiple severity requested).
    type: float
  siteId:
    description:
    - >
      SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) Examples `?siteId=id1` (single siteId
      requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple siteId requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (`*`) character search support. E.g. `*uuid*, *uuid, uuid*` Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyId requested).
    type: str
  networkDeviceName:
    description:
    - >
      NetworkDeviceName query parameter. Network device name. This parameter is applicable for network device
      related families. This field supports wildcard (`*`) character-based search. Ex `*Branch*` or `Branch*` or
      `*Branch` Examples `networkDeviceName=Branch-3-Gateway` (single networkDeviceName requested)
      `networkDeviceName=Branch-3-Gateway&networkDeviceName=Branch-3-Switch` (multiple networkDeviceName
      requested).
    type: str
  networkDeviceId:
    description:
    - >
      NetworkDeviceId query parameter. The list of Network Device Uuids. (Ex.
      `6bef213c-19ca-4170-8375-b694e251101c`) Examples `networkDeviceId=6bef213c-19ca-4170-8375-b694e251101c`
      (single networkDeviceId requested) `networkDeviceId=6bef213c-19ca-4170-8375-
      b694e251101c&networkDeviceId=32219612-819e-4b5e-a96b-cf22aca13dd9&networkDeviceId=2541e9a7-b80d-4955-8aa2-
      79b233318ba0` (multiple networkDeviceId with & separator).
    type: str
  apMac:
    description:
    - >
      ApMac query parameter. MAC address of the access point. This parameter is applicable for `Unified AP` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*50 0F*` or `50
      0F*` or `*50 0F` Examples `apMac=50 0F 80 0F F7 E0` (single apMac requested) `apMac=50 0F 80 0F F7
      E0&apMac=18 80 90 AB 7E A0` (multiple apMac requested).
    type: str
  clientMac:
    description:
    - >
      ClientMac query parameter. MAC address of the client. This parameter is applicable for `Wired Client` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*66 2B*` or `66
      2B*` or `*66 2B` Examples `clientMac=66 2B B8 D2 01 56` (single clientMac requested) `clientMac=66 2B B8 D2
      01 56&clientMac=DC A6 32 F5 5A 89` (multiple clientMac requested).
    type: str
  attribute:
    description:
    - >
      Attribute query parameter. The list of attributes that needs to be included in the response. If this
      parameter is not provided, then basic attributes (`id`, `name`, `timestamp`, `details`, `messageType`,
      `siteHierarchyId`, `siteHierarchy`, `deviceFamily`, `networkDeviceId`, `networkDeviceName`,
      `managementIpAddress`) would be part of the response. Examples `attribute=name` (single attribute requested)
      `attribute=name&attribute=networkDeviceName` (multiple attribute requested).
    type: str
  view:
    description:
    - >
      View query parameter. The list of events views. Please refer to `EventViews` for the supported list Examples
      `view=network` (single view requested) `view=network&view=ap` (multiple view requested).
    type: str
  offset:
    description:
    - >
      Offset query parameter. Specifies the starting point within all records returned by the API. It's one based
      offset. The starting value is 1.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
  sortBy:
    description:
    - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
    - Order query parameter. The sort order of the field ascending or descending.
    type: str
  id:
    description:
    - Id path parameter. Unique identifier for the event.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetDetailsOfASingleAssuranceEventV1
  description: Complete reference of the GetDetailsOfASingleAssuranceEventV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-details-of-a-single-assurance-event
- name: Cisco DNA Center documentation for Devices QueryAssuranceEventsV1
  description: Complete reference of the QueryAssuranceEventsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!query-assurance-events
notes:
  - SDK Method used are
    devices.Devices.get_details_of_a_single_assurance_event_v1,
    devices.Devices.query_assurance_events_v1,

  - Paths used are
    get /dna/data/api/v1/assuranceEvents,
    get /dna/data/api/v1/assuranceEvents/{id},

"""

EXAMPLES = r"""
- name: Get all Assurance Events V1
  cisco.dnac.assurance_events_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceFamily: string
    startTime: 0
    endTime: 0
    messageType: string
    severity: 0
    siteId: string
    siteHierarchyId: string
    networkDeviceName: string
    networkDeviceId: string
    apMac: string
    clientMac: string
    attribute: string
    view: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result

- name: Get Assurance Events V1 by id
  cisco.dnac.assurance_events_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    attribute: string
    view: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "oldRadioChannelWidth": "string",
        "clientMac": "string",
        "switchNumber": "string",
        "assocRssi": 0,
        "affectedClients": [
          "string"
        ],
        "isPrivateMac": true,
        "frequency": "string",
        "apRole": "string",
        "replacingDeviceSerialNumber": "string",
        "messageType": "string",
        "failureCategory": "string",
        "apSwitchName": "string",
        "apSwitchId": "string",
        "radioChannelUtilization": "string",
        "mnemonic": "string",
        "radioChannelSlot": 0,
        "details": "string",
        "id": "string",
        "lastApDisconnectReason": "string",
        "networkDeviceName": "string",
        "identifier": "string",
        "reasonDescription": "string",
        "vlanId": "string",
        "udnId": "string",
        "auditSessionId": "string",
        "apMac": "string",
        "deviceFamily": "string",
        "radioNoise": "string",
        "wlcName": "string",
        "apRadioOperationState": "string",
        "name": "string",
        "failureIpAddress": "string",
        "newRadioChannelList": "string",
        "duid": "string",
        "roamType": "string",
        "candidateAPs": [
          {
            "apId": "string",
            "apName": "string",
            "apMac": "string",
            "bssid": "string",
            "rssi": 0
          }
        ],
        "replacedDeviceSerialNumber": "string",
        "oldRadioChannelList": "string",
        "ssid": "string",
        "subReasonDescription": "string",
        "wirelessClientEventEndTime": 0,
        "ipv4": "string",
        "wlcId": "string",
        "ipv6": "string",
        "missingResponseAPs": [
          {
            "apId": "string",
            "apName": "string",
            "apMac": "string",
            "bssid": "string",
            "type": "string",
            "frameType": "string"
          }
        ],
        "timestamp": 0,
        "severity": 0,
        "currentRadioPowerLevel": 0,
        "newRadioChannelWidth": "string",
        "assocSnr": 0,
        "authServerIp": "string",
        "childEvents": [
          {
            "id": "string",
            "name": "string",
            "timestamp": 0,
            "wirelessEventType": 0,
            "details": "string",
            "reasonCode": "string",
            "reasonDescription": "string",
            "subReasonCode": "string",
            "subReasonDescription": "string",
            "resultStatus": "string",
            "failureCategory": "string"
          }
        ],
        "connectedInterfaceName": "string",
        "dhcpServerIp": "string",
        "managementIpAddress": "string",
        "previousRadioPowerLevel": 0,
        "resultStatus": "string",
        "radioInterference": "string",
        "networkDeviceId": "string",
        "siteHierarchy": "string",
        "eventStatus": "string",
        "wirelessClientEventStartTime": 0,
        "siteHierarchyId": "string",
        "udnName": "string",
        "facility": "string",
        "lastApResetType": "string",
        "invalidIeAPs": [
          {
            "apId": "string",
            "apName": "string",
            "apMac": "string",
            "bssid": "string",
            "type": "string",
            "frameType": "string",
            "ies": "string"
          }
        ],
        "username": "string"
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the iosxr_logging_global module
"""


class Logging_globalArgs(object):  # pylint: disable=R0903
    """The arg spec for the iosxr_logging_global module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "type": "dict",
            "options": {
                "archive": {
                    "type": "dict",
                    "options": {
                        "device": {"type": "str"},
                        "archive_length": {"type": "int"},
                        "archive_size": {"type": "int"},
                        "file_size": {"type": "int"},
                        "frequency": {
                            "type": "str",
                            "choices": ["daily", "weekly"],
                        },
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "threshold": {"type": "int"},
                    },
                },
                "buffered": {
                    "type": "dict",
                    "options": {
                        "size": {"type": "int"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "discriminator": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "match_params": {
                                    "type": "str",
                                    "choices": [
                                        "match1",
                                        "match2",
                                        "match3",
                                        "nomatch1",
                                        "nomatch2",
                                        "nomatch3",
                                    ],
                                },
                                "name": {"type": "str"},
                            },
                        },
                    },
                },
                "console": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warning",
                            ],
                        },
                        "discriminator": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "match_params": {
                                    "type": "str",
                                    "choices": [
                                        "match1",
                                        "match2",
                                        "match3",
                                        "nomatch1",
                                        "nomatch2",
                                        "nomatch3",
                                    ],
                                },
                                "name": {"type": "str"},
                            },
                        },
                    },
                },
                "correlator": {
                    "type": "dict",
                    "options": {
                        "buffer_size": {"type": "int"},
                        "rules": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "rule_name": {"type": "str"},
                                "rule_type": {
                                    "type": "str",
                                    "choices": ["stateful", "nonstateful"],
                                },
                                "timeout": {"type": "int"},
                                "timeout_rootcause": {"type": "int"},
                                "context_correlation": {"type": "bool"},
                                "reissue_nonbistate": {"type": "bool"},
                                "reparent": {"type": "bool"},
                            },
                        },
                        "rule_sets": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "name": {"type": "str"},
                                "rulename": {
                                    "type": "list",
                                    "elements": "str",
                                },
                            },
                        },
                    },
                },
                "events": {
                    "type": "dict",
                    "options": {
                        "buffer_size": {"type": "int"},
                        "display_location": {"type": "bool"},
                        "filter_match": {"type": "list", "elements": "str"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "threshold": {"type": "int"},
                    },
                },
                "facility": {
                    "type": "str",
                    "choices": [
                        "auth",
                        "cron",
                        "daemon",
                        "kern",
                        "local0",
                        "local1",
                        "local2",
                        "local3",
                        "local4",
                        "local5",
                        "local6",
                        "local7",
                        "lpr",
                        "mail",
                        "news",
                        "sys10",
                        "sys11",
                        "sys12",
                        "sys13",
                        "sys14",
                        "sys9",
                        "syslog",
                        "user",
                        "uucp",
                    ],
                },
                "files": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "name": {"type": "str"},
                        "path": {"type": "str"},
                        "maxfilesize": {"type": "int"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "info",
                                "notifications",
                                "warning",
                            ],
                        },
                    },
                },
                "format": {"type": "bool"},
                "history": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "size": {"type": "int"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                    },
                },
                "hostnameprefix": {"type": "str"},
                "ipv4": {
                    "type": "dict",
                    "options": {
                        "dscp": {"type": "str"},
                        "precedence": {"type": "str"},
                    },
                },
                "ipv6": {
                    "type": "dict",
                    "options": {
                        "dscp": {"type": "str"},
                        "precedence": {"type": "str"},
                    },
                },
                "localfilesize": {"type": "int"},
                "monitor": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "discriminator": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "match_params": {
                                    "type": "str",
                                    "choices": [
                                        "match1",
                                        "match2",
                                        "match3",
                                        "nomatch1",
                                        "nomatch2",
                                        "nomatch3",
                                    ],
                                },
                                "name": {"type": "str"},
                            },
                        },
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warning",
                            ],
                        },
                    },
                },
                "source_interfaces": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "interface": {"type": "str"},
                        "vrf": {"type": "str"},
                    },
                },
                "suppress": {
                    "type": "dict",
                    "options": {
                        "apply_rule": {"type": "str"},
                        "duplicates": {"type": "bool"},
                    },
                },
                "tls_servers": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "name": {"type": "str"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "tls_hostname": {"type": "str"},
                        "trustpoint": {"type": "str"},
                        "vrf": {"type": "str"},
                    },
                },
                "trap": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warning",
                            ],
                        },
                    },
                },
                "hosts": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "error",
                                "info",
                                "notifications",
                                "warning",
                            ],
                        },
                        "host": {"type": "str"},
                        "port": {"type": "str", "default": "default"},
                        "vrf": {"type": "str", "default": "default"},
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "parsed",
                "rendered",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: assurance_issues_query_count_v1
short_description: Resource module for Assurance Issues Query Count V1
description:
- Manage operation create of the resource Assurance Issues Query Count V1.
- >
   Returns the total number issues for given set of filters. If there is no start and/or end time, then end time will
   be defaulted to current time and start time will be defaulted to 24-hours ago from end time. For detailed
   information about the usage of the API, please refer to the Open API specification document - https
   //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   IssuesList-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  endTime:
    description: End Time.
    type: int
  filters:
    description: Assurance Issues Query Count's filters.
    elements: dict
    suboptions:
      filters:
        description: Assurance Issues Query Count's filters.
        elements: dict
        suboptions:
          key:
            description: Key.
            type: str
          operator:
            description: Operator.
            type: str
          value:
            description: Value.
            type: str
        type: list
      key:
        description: Key.
        type: str
      logicalOperator:
        description: Logical Operator.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description: Start Time.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues GetTheTotalNumberOfIssuesForGivenSetOfFiltersV1
  description: Complete reference of the GetTheTotalNumberOfIssuesForGivenSetOfFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-total-number-of-issues-for-given-set-of-filters
notes:
  - SDK Method used are
    issues.Issues.get_the_total_number_of_issues_for_given_set_of_filters_v1,

  - Paths used are
    post /dna/data/api/v1/assuranceIssues/query/count,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.assurance_issues_query_count_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    endTime: 0
    filters:
    - filters:
      - key: string
        operator: string
        value: string
      key: string
      logicalOperator: string
      operator: string
      value: string
    headers: '{{my_headers | from_json}}'
    startTime: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "filters": [
        {
          "key": "string",
          "value": "string",
          "operator": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: wireless_provision_device_create
short_description: Resource module for Wireless Provision Device Create
description:
- This module represents an alias of the module wireless_provision_device_create_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Wireless Provision Device Create's payload.
    elements: dict
    suboptions:
      deviceName:
        description: Controller Name.
        type: str
      dynamicInterfaces:
        description: Wireless Provision Device Create's dynamicInterfaces.
        elements: dict
        suboptions:
          interfaceGateway:
            description: Interface Gateway. Required for AireOS.
            type: str
          interfaceIPAddress:
            description: Interface IP Address. Required for AireOS.
            type: str
          interfaceName:
            description: Interface Name. Required for both AireOS and EWLC.
            type: str
          interfaceNetmaskInCIDR:
            description: Interface Netmask In CIDR. Required for AireOS.
            type: int
          lagOrPortNumber:
            description: Lag Or Port Number. Required for AireOS.
            type: int
          vlanId:
            description: VLAN ID. Required for both AireOS and EWLC.
            type: int
        type: list
      managedAPLocations:
        description: List of managed AP locations (Site Hierarchies).
        elements: str
        type: list
      site:
        description: Full Site Hierarchy where device has to be assigned.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless ProvisionV1
  description: Complete reference of the ProvisionV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!provision
notes:
  - SDK Method used are
    wireless.Wireless.provision_v1,

  - Paths used are
    post /dna/intent/api/v1/wireless/provision,
  - It should be noted that this module is an alias of wireless_provision_device_create_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_provision_device_create:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    payload:
    - deviceName: string
      dynamicInterfaces:
      - interfaceGateway: string
        interfaceIPAddress: string
        interfaceName: string
        interfaceNetmaskInCIDR: 0
        lagOrPortNumber: 0
        vlanId: 0
      managedAPLocations:
      - string
      site: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: tag_membership_v1
short_description: Resource module for Tag Membership V1
description:
- Manage operation update of the resource Tag Membership V1.
- >
   Update tag membership. As part of the request payload through this API, only the specified members are added /
   retained to the given input tags. Possible values of memberType attribute in the request payload can be queried by
   using the /tag/member/type API.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  memberToTags:
    description: Tag Membership's memberToTags.
    suboptions:
      key:
        description: Tag Membership's key.
        elements: str
        type: list
    type: dict
  memberType:
    description: Tag Membership's memberType.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Tag UpdateTagMembershipV1
  description: Complete reference of the UpdateTagMembershipV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-tag-membership
notes:
  - SDK Method used are
    tag.Tag.update_tag_membership_v1,

  - Paths used are
    put /dna/intent/api/v1/tag/member,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.tag_membership_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    memberToTags:
      key:
      - string
    memberType: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "taskId": "string",
        "url": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino), Jurijs Fadejevs (@needgithubid)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_azurefirewall
version_added: '0.1.2'
short_description: Manage Azure Firewall instance
description:
    - Create, update and delete instance of Azure Firewall.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    name:
        description:
            - The name of the Azure Firewall.
        required: true
        type: str
    location:
        description:
            - Resource location.
        type: str
    application_rule_collections:
        description:
            - Collection of application rule collections used by Azure Firewall.
        type: list
        elements: dict
        suboptions:
            priority:
                description:
                    - Priority of the application rule collection resource.
                type: int
            action:
                description:
                    - The action type of a rule collection.
                choices:
                    - allow
                    - deny
                type: str
            rules:
                description:
                    - Collection of rules used by a application rule collection.
                type: list
                elements: raw
                suboptions:
                    name:
                        description:
                            - Name of the application rule.
                        type: str
                    description:
                        description:
                            - Description of the rule.
                        type: str
                    source_addresses:
                        description:
                            - List of source IP addresses for this rule.
                        type: list
                        elements: str
                    protocols:
                        description:
                            - Array of ApplicationRuleProtocols.
                        elements: dict
                        type: list
                        suboptions:
                            type:
                                description:
                                    - The type of the protocols.
                                type: str
                            port:
                                description:
                                    - The ports of the protocols.
                                type: str
                    target_fqdns:
                        description:
                            - List of FQDNs for this rule.
                        type: list
                        elements: raw
                    fqdn_tags:
                        description:
                            - List of FQDN Tags for this rule.
                        type: list
                        elements: raw
            name:
                description:
                    - Gets name of the resource that is unique within a resource group.
                    - This name can be used to access the resource.
                type: str
    nat_rule_collections:
        description:
            - Collection of NAT rule collections used by Azure Firewall.
        type: list
        elements: dict
        suboptions:
            priority:
                description:
                    - Priority of the NAT rule collection resource.
                type: int
            action:
                description:
                    - The action type of a NAT rule collection
                choices:
                    - snat
                    - dnat
                type: str
            rules:
                description:
                    - Collection of rules used by a NAT rule collection.
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - Name of the NAT rule.
                        type: str
                    description:
                        description:
                            - Description of the rule.
                        type: str
                    source_addresses:
                        description:
                            - List of source IP addresses for this rule.
                        type: list
                        elements: str
                    destination_addresses:
                        description:
                            - List of destination IP addresses for this rule.
                        type: list
                        elements: str
                    destination_ports:
                        description:
                            - List of destination ports.
                        type: list
                        elements: str
                    protocols:
                        description:
                            - Array of AzureFirewallNetworkRuleProtocols applicable to this NAT rule.
                        type: list
                        elements: raw
                    translated_address:
                        description:
                            - The translated address for this NAT rule.
                        type: str
                    translated_port:
                        description:
                            - The translated port for this NAT rule.
                        type: str
            name:
                description:
                    - Gets name of the resource that is unique within a resource group.
                    - This name can be used to access the resource.
                type: str
    network_rule_collections:
        description:
            - Collection of network rule collections used by Azure Firewall.
        type: list
        elements: dict
        suboptions:
            priority:
                description:
                    - Priority of the network rule collection resource.
                type: int
            action:
                description:
                    - The action type of a rule collection.
                type: str
                choices:
                    - allow
                    - deny
            rules:
                description:
                    - Collection of rules used by a network rule collection.
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - Name of the network rule.
                        type: str
                    description:
                        description:
                            - Description of the rule.
                        type: str
                    protocols:
                        description:
                            - Array of AzureFirewallNetworkRuleProtocols.
                        type: list
                        elements: raw
                    source_addresses:
                        description:
                            - List of source IP addresses for this rule.
                        type: list
                        elements: str
                    destination_addresses:
                        description:
                            - List of destination IP addresses.
                        type: list
                        elements: str
                    destination_ports:
                        description:
                            - List of destination ports.
                        type: list
                        elements: str
            name:
                description:
                    - Gets name of the resource that is unique within a resource group.
                    - This name can be used to access the resource.
                type: str
    ip_configurations:
        description:
            - IP configuration of the Azure Firewall resource.
        type: list
        elements: dict
        suboptions:
            subnet:
                description:
                    - Existing subnet.
                    - It can be a string containing subnet resource ID.
                    - It can be a dictionary containing I(name), I(virtual_network_name) and optionally I(resource_group) .
                type: raw
            public_ip_address:
                description:
                    - Existing public IP address.
                    - It can be a string containing resource ID.
                    - It can be a string containing a name in current resource group.
                    - It can be a dictionary containing I(name) and optionally I(resource_group).
                type: raw
            name:
                description:
                    - Name of the resource that is unique within a resource group.
                    - This name can be used to access the resource.
                type: str
    state:
        description:
            - Assert the state of the AzureFirewall.
            - Use C(present) to create or update an AzureFirewall and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Zim Kalinowski (@zikalino)
    - Jurijs Fadejevs (@needgithubid)

'''

EXAMPLES = '''
- name: Create Azure Firewall
  azure_rm_azurefirewall:
    resource_group: myResourceGroup
    name: myAzureFirewall
    tags:
      key1: value1
    application_rule_collections:
      - priority: 110
        action: deny
        rules:
          - name: rule1
            description: Deny inbound rule
            source_addresses:
              - 216.58.216.164
              - 10.0.0.0/24
            protocols:
              - type: https
                port: '443'
            target_fqdns:
              - www.test.com
        name: apprulecoll
    nat_rule_collections:
      - priority: 112
        action: dnat
        rules:
          - name: DNAT-HTTPS-traffic
            description: D-NAT all outbound web traffic for inspection
            source_addresses:
              - '*'
            destination_addresses:
              - 1.2.3.4
            destination_ports:
              - '443'
            protocols:
              - tcp
            translated_address: 1.2.3.5
            translated_port: '8443'
        name: natrulecoll
    network_rule_collections:
      - priority: 112
        action: deny
        rules:
          - name: L4-traffic
            description: Block traffic based on source IPs and ports
            protocols:
              - tcp
            source_addresses:
              - 192.168.1.1-192.168.1.12
              - 10.1.4.12-10.1.4.255
            destination_addresses:
              - '*'
            destination_ports:
              - 443-444
              - '8443'
        name: netrulecoll
    ip_configurations:
      - subnet: >-
          /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup
          /providers/Microsoft.Network/virtualNetworks/myVirtualNetwork
          /subnets/AzureFirewallSubnet
        public_ip_address: >-
          /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup
          /providers/Microsoft.Network/publicIPAddresses/
          myPublicIpAddress
        name: azureFirewallIpConfiguration
- name: Delete Azure Firewall
  azure_rm_azurefirewall:
    resource_group: myResourceGroup
    name: myAzureFirewall
    state: absent
'''

RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/azureFirewalls/myAzureFirewall
'''

import time
import json
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMAzureFirewalls(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str',
            ),
            application_rule_collections=dict(
                type='list',
                elements='dict',
                options=dict(
                    priority=dict(
                        type='int',
                    ),
                    action=dict(
                        type='str',
                        choices=['allow',
                                 'deny'],
                    ),
                    rules=dict(
                        type='list',
                        elements='raw',
                        options=dict(
                            name=dict(
                                type='str'
                            ),
                            description=dict(
                                type='str'
                            ),
                            source_addresses=dict(
                                type='list',
                                elements='str',
                            ),
                            protocols=dict(
                                type='list',
                                elements='dict',
                                options=dict(
                                    type=dict(
                                        type='str',
                                    ),
                                    port=dict(
                                        type='str'
                                    )
                                )
                            ),
                            target_fqdns=dict(
                                type='list',
                                elements='raw',
                            ),
                            fqdn_tags=dict(
                                type='list',
                                elements='raw',
                            )
                        )
                    ),
                    name=dict(
                        type='str'
                    )
                )
            ),
            nat_rule_collections=dict(
                type='list',
                elements='dict',
                options=dict(
                    priority=dict(
                        type='int',
                    ),
                    action=dict(
                        type='str',
                        choices=['snat',
                                 'dnat'],
                    ),
                    rules=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            name=dict(
                                type='str'
                            ),
                            description=dict(
                                type='str'
                            ),
                            source_addresses=dict(
                                type='list',
                                elements='str',
                            ),
                            destination_addresses=dict(
                                type='list',
                                elements='str',
                            ),
                            destination_ports=dict(
                                type='list',
                                elements='str',
                            ),
                            protocols=dict(
                                type='list',
                                elements='raw'
                            ),
                            translated_address=dict(
                                type='str',
                            ),
                            translated_port=dict(
                                type='str',
                            )
                        )
                    ),
                    name=dict(
                        type='str'
                    )
                )
            ),
            network_rule_collections=dict(
                type='list',
                elements='dict',
                options=dict(
                    priority=dict(
                        type='int',
                    ),
                    action=dict(
                        type='str',
                        choices=['allow',
                                 'deny'],
                    ),
                    rules=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            name=dict(
                                type='str'
                            ),
                            description=dict(
                                type='str'
                            ),
                            protocols=dict(
                                type='list',
                                elements='raw'
                            ),
                            source_addresses=dict(
                                type='list',
                                elements='str',
                            ),
                            destination_addresses=dict(
                                type='list',
                                elements='str',
                            ),
                            destination_ports=dict(
                                type='list',
                                elements='str',
                            )
                        )
                    ),
                    name=dict(
                        type='str'
                    )
                )
            ),
            ip_configurations=dict(
                type='list',
                elements='dict',
                options=dict(
                    subnet=dict(
                        type='raw',
                    ),
                    public_ip_address=dict(
                        type='raw',
                    ),
                    name=dict(
                        type='str'
                    )
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.body = {}
        self.body['properties'] = {}

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200, 201, 202]
        self.to_do = Actions.NoAction

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2018-11-01'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(AzureRMAzureFirewalls, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                    supports_check_mode=True,
                                                    supports_tags=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == 'application_rule_collections':
                    self.body['properties']['applicationRuleCollections'] = []
                    for item in kwargs[key]:
                        app_rule = dict(properties={})
                        if item.get('priority') is not None:
                            app_rule['properties']['priority'] = item['priority']
                        if item.get('action') is not None:
                            app_rule['properties']['action'] = dict(type=item['action'])
                        if item.get('name') is not None:
                            app_rule['name'] = item['name']
                        if item.get('rules') is not None:
                            app_rule['properties']['rules'] = []
                            for value in item['rules']:
                                rule_value = {}
                                if value.get('name') is not None:
                                    rule_value['name'] = value['name']
                                if value.get('description') is not None:
                                    rule_value['description'] = value['description']
                                if value.get('source_addresses') is not None:
                                    rule_value['sourceAddresses'] = value.get('source_addresses')
                                if value.get('target_fqdns') is not None:
                                    rule_value['targetFqdns'] = value.get('target_fqdns')
                                if value.get('fqdn_tags') is not None:
                                    rule_value['fqdnTags'] = value.get('fqdn_tags')
                                if value.get('protocols') is not None:
                                    rule_value['protocols'] = []
                                    for pp in value['protocols']:
                                        pro = {}
                                        if pp.get('type') is not None:
                                            pro['protocolType'] = pp.get('type')
                                        if pp.get('port') is not None:
                                            pro['port'] = pp.get('port')
                                        rule_value['protocols'].append(pro)
                                app_rule['properties']['rules'].append(rule_value)
                        self.body['properties']['applicationRuleCollections'].append(app_rule)
                elif key == 'nat_rule_collections':
                    self.body['properties']['natRuleCollections'] = []
                    for item in kwargs[key]:
                        nat_rule = dict(properties={})
                        if item.get('priority') is not None:
                            nat_rule['properties']['priority'] = item['priority']
                        if item.get('action') is not None:
                            nat_rule['properties']['action'] = dict(type=item['action'])
                        if item.get('name') is not None:
                            nat_rule['name'] = item['name']
                        if item.get('rules') is not None:
                            nat_rule['properties']['rules'] = []
                            for value in item['rules']:
                                nat_value = {}
                                if value.get('name') is not None:
                                    nat_value['name'] = value.get('name')
                                if value.get('description') is not None:
                                    nat_value['description'] = value.get('description')
                                if value.get('source_addresses') is not None:
                                    nat_value['sourceAddresses'] = value.get('source_addresses')
                                if value.get('destination_addresses') is not None:
                                    nat_value['destinationAddresses'] = value.get('destination_addresses')
                                if value.get('destination_ports') is not None:
                                    nat_value['destinationPorts'] = value.get('destination_ports')
                                if value.get('protocols') is not None:
                                    nat_value['protocols'] = value.get('protocols')
                                if value.get('translated_address') is not None:
                                    nat_value['translatedAddress'] = value.get('translated_address')
                                if value.get('translated_port') is not None:
                                    nat_value['translatedPort'] = value.get('translated_port')
                                nat_rule['properties']['rules'].append(nat_value)
                        self.body['properties']['natRuleCollections'].append(nat_rule)
                elif key == 'network_rule_collections':
                    self.body['properties']['networkRuleCollections'] = []
                    for item in kwargs[key]:
                        network_rule = dict(properties={})
                        if item.get('priority') is not None:
                            network_rule['properties']['priority'] = item['priority']
                        if item.get('action') is not None:
                            network_rule['properties']['action'] = dict(type=item['action'])
                        if item.get('name') is not None:
                            network_rule['name'] = item['name']
                        if item.get('rules') is not None:
                            network_rule['properties']['rules'] = []
                            for value in item['rules']:
                                net_value = {}
                                if value.get('name') is not None:
                                    net_value['name'] = value.get('name')
                                if value.get('description') is not None:
                                    net_value['description'] = value.get('description')
                                if value.get('source_addresses') is not None:
                                    net_value['sourceAddresses'] = value.get('source_addresses')
                                if value.get('destination_addresses') is not None:
                                    net_value['destinationAddresses'] = value.get('destination_addresses')
                                if value.get('destination_ports') is not None:
                                    net_value['destinationPorts'] = value.get('destination_ports')
                                if value.get('protocols') is not None:
                                    net_value['protocols'] = value.get('protocols')
                                network_rule['properties']['rules'].append(net_value)
                        self.body['properties']['networkRuleCollections'].append(network_rule)
                elif key == 'ip_configurations':
                    self.body['properties']['ipConfigurations'] = []
                    for item in kwargs[key]:
                        ipconfig = dict(properties={})
                        if item.get('subnet') is not None:
                            ipconfig['properties']['subnet'] = {}
                            if isinstance(item['subnet'], str):
                                ipconfig['properties']['subnet']['id'] = item['subnet']
                            elif isinstance(item['subnet'], dict):
                                if item['subnet'].get('id') is not None:
                                    ipconfig['properties']['subnet']['id'] = item['subnet'].get('id')
                                elif (item['subnet'].get('resource_group') is not None and item['subnet'].get('name') is not None and
                                      item['subnet'].get('virtual_network_name') is not None):
                                    ipconfig['properties']['subnet']['id'] = ('/subscriptions/' +
                                                                              self.subscription_id +
                                                                              '/resourceGroups/' +
                                                                              item['subnet'].get('resource_group') +
                                                                              '/providers/Microsoft.Network/virtualNetworks/' +
                                                                              item['subnet'].get('virtual_network_name') +
                                                                              '/subnets/' +
                                                                              item['subnet'].get('name'))
                                elif item['subnet'].get('name') is not None and item['subnet'].get('virtual_network_name') is not None:
                                    ipconfig['properties']['subnet']['id'] = ('/subscriptions/' +
                                                                              self.subscription_id +
                                                                              '/resourceGroups/' +
                                                                              self.resource_group +
                                                                              '/providers/Microsoft.Network/virtualNetworks/' +
                                                                              item['subnet'].get('virtual_network_name') +
                                                                              '/subnets/' +
                                                                              item['subnet'].get('name'))
                                else:
                                    self.fail("The ip_configuration's subnet config error")
                            else:
                                self.fail("The ip_configuration's subnet config error")
                        if item.get('public_ip_address') is not None:
                            ipconfig['properties']['publicIPAddress'] = {}
                            if isinstance(item.get('public_ip_address'), str):
                                ipconfig['properties']['publicIPAddress']['id'] = item.get('public_ip_address')
                            elif isinstance(item.get('public_ip_address'), dict):
                                if item['public_ip_address'].get('id') is not None:
                                    ipconfig['properties']['publicIPAddress']['id'] = item['public_ip_address'].get('id')
                                elif item['public_ip_address'].get('resource_group') is not None and item['public_ip_address'].get('name') is not None:
                                    ipconfig['properties']['publicIPAddress']['id'] = ('/subscriptions/' +
                                                                                       self.subscription_id +
                                                                                       '/resourceGroups/' +
                                                                                       item['public_ip_address'].get('resource_group') +
                                                                                       '/providers/Microsoft.Network/publicIPAddresses/' +
                                                                                       item['public_ip_address'].get('name'))
                                elif item['public_ip_address'].get('name') is not None:
                                    ipconfig['properties']['publicIPAddress']['id'] = ('/subscriptions/' +
                                                                                       self.subscription_id +
                                                                                       '/resourceGroups/' +
                                                                                       self.resource_group +
                                                                                       '/providers/Microsoft.Network/publicIPAddresses/' +
                                                                                       item['public_ip_address'].get('name'))
                                else:
                                    self.fail("The ip_configuration's public ip address config error")
                            else:
                                self.fail("The ip_configuration's public ip address config error")

                        if item.get('name') is not None:
                            ipconfig['name'] = item['name']
                        self.body['properties']['ipConfigurations'].append(ipconfig)
                else:
                    self.body[key] = kwargs[key]

        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        resource_group = self.get_resource_group(self.resource_group)

        if 'location' not in self.body:
            self.body['location'] = resource_group.location

        self.url = ('/subscriptions' +
                    '/' + self.subscription_id +
                    '/resourceGroups' +
                    '/' + self.resource_group +
                    '/providers' +
                    '/Microsoft.Network' +
                    '/azureFirewalls' +
                    '/' + self.name)

        old_response = self.get_resource()

        if not old_response:
            self.log("AzureFirewall instance doesn't exist")

            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log('AzureFirewall instance already exists')

            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                update_tags, new_tags = self.update_tags(old_response.get('tags'))
                if update_tags:
                    self.to_do = Actions.Update
                    self.body['tags'] = new_tags

                if not self.default_compare({}, self.body, old_response, '', dict(compare=[])):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log('Need to Create / Update the AzureFirewall instance')

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_update_resource()

            # if not old_response:
            self.results['changed'] = True
            # else:
            #     self.results['changed'] = old_response.__ne__(response)
            self.log('Creation / Update done')
        elif self.to_do == Actions.Delete:
            self.log('AzureFirewall instance deleted')
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_resource()

            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_resource():
                time.sleep(20)
        else:
            self.log('AzureFirewall instance unchanged')
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]
            while response['properties']['provisioningState'] == 'Updating':
                time.sleep(30)
                response = self.get_resource()

        return self.results

    def create_update_resource(self):
        # self.log('Creating / Updating the AzureFirewall instance {0}'.format(self.))

        try:
            response = self.mgmt_client.query(self.url,
                                              'PUT',
                                              self.query_parameters,
                                              self.header_parameters,
                                              self.body,
                                              self.status_code,
                                              600,
                                              30)
        except Exception as exc:
            self.log('Error attempting to create the AzureFirewall instance.')
            self.fail('Error creating the AzureFirewall instance: {0}'.format(str(exc)))

        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response

    def delete_resource(self):
        # self.log('Deleting the AzureFirewall instance {0}'.format(self.))
        try:
            response = self.mgmt_client.query(self.url,
                                              'DELETE',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
        except Exception as e:
            self.log('Error attempting to delete the AzureFirewall instance.')
            self.fail('Error deleting the AzureFirewall instance: {0}'.format(str(e)))

        return True

    def get_resource(self):
        # self.log('Checking if the AzureFirewall instance {0} is present'.format(self.))
        found = False
        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            response = json.loads(response.body())
            found = True
            self.log("Response : {0}".format(response))
            # self.log("AzureFirewall instance : {0} found".format(response.name))
        except Exception as e:
            self.log('Did not find the AzureFirewall instance.')
        if found is True:
            return response

        return False


def main():
    AzureRMAzureFirewalls()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_networkflowlog_info

version_added: "2.5.0"

short_description: Get or list the network flow logs

description:
    - Get or list the network flow logs facts.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: true
    network_watcher_name:
        description:
            - Name of the network watcher.
        type: str
        required: true
    name:
        description:
            - Name of the network flow logs.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get the network watcher facts
  azure_rm_networkflowlog_info:
    resource_group: myResourceGroup
    network_watcher_name: mywatcher01
    name: flowlogname

- name: list the network flow logs and filter by tags
  azure_rm_networkflowlog_info:
    resource_group: myResourceGroup
    network_watcher_name: mywatcher01
    tags:
      - key1
'''

RETURN = '''
flow_logs:
    description:
        - The facts of the network flow logs.
    returned: always
    type: complex
    contains:
        resource_group:
            description:
                - The resource group.
            type: str
            returned: always
            sample: NetworkWatcherRG
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/NetWatcherRG/providers/Microsoft.Network/networkWatchers/NetWatcher_eastus/flowLogs/xz-flowlog"
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: xz-flowlog
        network_watcher_name:
            description:
                - The name of the network watcher.
            type: str
            returned: always
            sample: NetWatcher_eastus
        target_resource_id:
            description:
                - ID of network security group to which flow log will be applied.
            type: str
            returned: always
            sample: /subscriptions/xxx-xxx/resourceGroups/xz3mlwaiserv/providers/Microsoft.Network/virtualNetworks/xz3mlwvnet"
        storage_id:
            description:
                - ID of the storage account which is used to store the flow log.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/AutoTagFunctionAppRG/providers/Microsoft.Storage/storageAccounts/autotagfunc01"
        enanbled:
            description:
                - Flag to enable/disable flow logging.
            type: str
            returned: always
            sample: true
        retention_policy:
            description:
                - Parameters that define the retention policy for flow log.
            type: complex
            returned: always
            contains:
                day:
                    description:
                        - Number of days to retain flow log records.
                    type: int
                    returned: always
                    sample: 0
                enabled:
                    description:
                        - Flag to enable/disable retention.
                    type: bool
                    returned: always
                    sample: false
        flow_analytics_configuration:
            description:
                - Parameters that define the configuration of traffic analytics.
            type: complex
            returned: always
            contains:
                network_watcher_flow_analytics_configuration:
                    description:
                        - Parameters that define the configuration of traffic analytics.
                    type: complex
                    returned: always
                    contains:
                        enabled:
                            description:
                                - Flag to enable/disable traffic analytics.
                            type: bool
                            returned: always
                            sample: true
                        workspace_id:
                            description:
                                - The resource guid of the attached workspace.
                            type: str
                            returned: always
                            sample: 7c16a8dd-b983-4f75-b78b-a804c169306c
                        workspace_region:
                            description:
                                - The location of the attached workspace.
                            type: str
                            returned: always
                            sample: eastus
                        workspace_resource_id:
                            description:
                                - Resource Id of the attached workspace.
                            type: str
                            returned: always
                            sample: /subscriptions/xxx-xxx/resourceGroups/DefaulUS/providers/Microsoft.OperationalInsights/workspaces/DefaultWorkspace-0-EUS"
                        traffic_analytics_interval:
                            description:
                                - The interval in minutes which would decide how frequently TA service should do flow analytics.
                            type: str
                            returned: always
                            sample: 60
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'key1':'value1' }
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: "Microsoft.Network/networkWatchers/flowLogs"
        provisioning_state:
            description:
                - The provisioning state of the network flow logs resource.
            type: str
            returned: always
            sample: Succeeded
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMNetworkFlowLogInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            network_watcher_name=dict(type='str', required=True),
            name=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        self.results = dict(
            changed=False,
            flow_logs=[]
        )

        self.resource_group = None
        self.network_watcher_name = None
        self.name = None
        self.tags = None

        super(AzureRMNetworkFlowLogInfo, self).__init__(self.module_arg_spec,
                                                        supports_check_mode=True,
                                                        supports_tags=False,
                                                        facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            response = [self.get_by_name()]
        else:
            response = self.list_by_network_watcher()

        self.results['flow_logs'] = [self.to_dict(item) for item in response if response is not None]

        return self.results

    def get_by_name(self):
        response = None
        try:
            response = self.network_client.flow_logs.get(self.resource_group, self.network_watcher_name, self.name)

        except ResourceNotFoundError as exec:
            self.log("Failed to get network flow logs, Exception as {0}".format(exec))

        return response

    def list_by_network_watcher(self):
        response = []
        try:
            response = self.network_client.flow_logs.list(self.resource_group, self.network_watcher_name)
        except Exception as exec:
            self.log("Faild to list network flow logs by network watcher, exception as {0}".format(exec))
        return response

    def to_dict(self, body):
        results = dict()
        if body is not None and self.has_tags(body.tags, self.tags):
            results = dict(
                resource_group=self.resource_group,
                network_watcher_name=self.network_watcher_name,
                id=body.id,
                name=body.name,
                location=body.location,
                tags=body.tags,
                type=body.type,
                provisioning_state=body.provisioning_state,
                target_resource_id=body.target_resource_id,
                storage_id=body.storage_id,
                enabled=body.enabled,
                retention_policy=dict(),
                flow_analytics_configuration=dict()
            )
            if body.retention_policy is not None:
                results['retention_policy']['days'] = body.retention_policy.days
                results['retention_policy']['enabled'] = body.retention_policy.enabled
            if body.flow_analytics_configuration is not None:
                results['flow_analytics_configuration']['network_watcher_flow_analytics_configuration'] = dict()
                if body.flow_analytics_configuration.network_watcher_flow_analytics_configuration is not None:
                    new_config = body.flow_analytics_configuration.network_watcher_flow_analytics_configuration
                    results['flow_analytics_configuration']['network_watcher_flow_analytics_configuration']['enabled'] = new_config.enabled
                    results['flow_analytics_configuration']['network_watcher_flow_analytics_configuration']['workspace_id'] = new_config.workspace_id
                    results['flow_analytics_configuration']['network_watcher_flow_analytics_configuration']['workspace_region'] = new_config.workspace_region
                    results['flow_analytics_configuration']['network_watcher_flow_analytics_configuration']['workspace_resource_id'] = \
                        new_config.workspace_resource_id
                    results['flow_analytics_configuration']['network_watcher_flow_analytics_configuration']['traffic_analytics_interval'] = \
                        new_config.traffic_analytics_interval

        return results


def main():
    AzureRMNetworkFlowLogInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
module: backup_selection
short_description: Create, delete and modify AWS Backup selection
version_added: 6.0.0
description:
  - Manages AWS Backup selections.
  - For more information see the AWS documentation for backup selections
    U(https://docs.aws.amazon.com/aws-backup/latest/devguide/assigning-resources.html).
options:
  backup_plan_name:
    description:
      - Uniquely identifies the backup plan to be associated with the selection of resources.
    required: true
    type: str
    aliases:
      - plan_name
  backup_selection_name:
    description:
      - The display name of a resource selection document. Must contain 1 to 50 alphanumeric or '-_.' characters.
    required: true
    type: str
    aliases:
      - selection_name
  iam_role_arn:
    description:
      - The ARN of the IAM role that Backup uses to authenticate when backing up the target resource.
    type: str
  resources:
    description:
      - A list of Amazon Resource Names (ARNs) to assign to a backup plan. The maximum number of ARNs is 500 without wildcards,
        or 30 ARNs with wildcards. If you need to assign many resources to a backup plan, consider a different resource selection
        strategy, such as assigning all resources of a resource type or refining your resource selection using tags.
    type: list
    elements: str
  list_of_tags:
    description:
      - A list of conditions that you define to assign resources to your backup plans using tags.
      - Condition operators are case sensitive.
      - When you specify more than one condition in O(list_of_tags), you assign all resources that match AT LEAST ONE condition (using OR logic).
    type: list
    elements: dict
    suboptions:
        condition_type:
            description:
            - An operation applied to a key-value pair used to assign resources to your backup plan.
            - Condition only supports V(STRINGEQUALS).
            type: str
        condition_key:
            description:
            - The key in a key-value pair.
            type: str
        condition_value:
            description:
            - The value in a key-value pair.
            type: str
  not_resources:
    description:
      - A list of Amazon Resource Names (ARNs) to exclude from a backup plan. The maximum number of ARNs is 500 without wildcards,
        or 30 ARNs with wildcards. If you need to exclude many resources from a backup plan, consider a different resource
        selection strategy, such as assigning only one or a few resource types or refining your resource selection using tags.
    type: list
    elements: str
  conditions:
    description:
      - A list of conditions (expressed as a dict) that you define to assign resources to your backup plans using tags.
      - When you specify more than one condition in I(conditions), you only assign the resources that match ALL conditions (using AND logic).
      - O(conditions) supports V(string_equals), V(string_like), V(string_not_equals), and V(string_not_like). O(list_of_tags) only supports V(string_equals).
    type: dict
    suboptions:
      string_equals:
        description:
          - Filters the values of your tagged resources for only those resources that you tagged with the same value.
        type: list
        default: []
        elements: dict
        suboptions:
          condition_key:
            description:
              - The key in a key-value pair.
              - O(conditions.string_equals.condition_key) in the O(conditions) option must use the AWS resource tag prefix, e.g. 'aws:ResourceTag/key-name'
            type: str
          condition_value:
            description: The value in a key-value pair.
            type: str
      string_like:
        description:
          - Filters the values of your tagged resources for matching tag values with the use of a wildcard character (*) anywhere in the string.
            For example, "prod*" or "*rod*" matches the tag value "production".
        type: list
        default: []
        elements: dict
        suboptions:
          condition_key:
            description:
              - The key in a key-value pair.
              - O(conditions.string_like.condition_key) in the O(conditions) option must use the AWS resource tag prefix, e.g. 'aws:ResourceTag/key-name'
            type: str
          condition_value:
            description: The value in a key-value pair.
            type: str
      string_not_equals:
        description:
          - Filters the values of your tagged resources for only those resources that you tagged that do not have the same value.
        type: list
        default: []
        elements: dict
        suboptions:
          condition_key:
            description:
              - The key in a key-value pair.
              - O(conditions.string_not_equals.condition_key) in the O(conditions) option must use the AWS resource tag prefix, e.g. 'aws:ResourceTag/key-name'
            type: str
          condition_value:
            description: The value in a key-value pair.
            type: str
      string_not_like:
        description:
          - Filters the values of your tagged resources for non-matching tag values with the use of a wildcard character (*) anywhere in the string.
        type: list
        default: []
        elements: dict
        suboptions:
          condition_key:
            description:
              - The key in a key-value pair.
              - O(conditions.string_not_like.condition_key) in the O(conditions) option must use the AWS resource tag prefix, e.g. 'aws:ResourceTag/key-name'
            type: str
          condition_value:
            description: The value in a key-value pair.
            type: str
  state:
    description:
      - Create, delete a backup selection.
    default: present
    choices: ['present', 'absent']
    type: str
author:
  - Kristof Imre Szabo (@krisek)
  - Alina Buzachis (@alinabuzachis)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""


EXAMPLES = r"""
- name: Create backup selection
  amazon.aws.backup_selection:
    selection_name: elastic
    backup_plan_name: test-backup-plan
    iam_role_arn: arn:aws:iam::111122223333:role/system-backup
    resources:
      - arn:aws:elasticfilesystem:*:*:file-system/*

- name: Create backup selection, assign resources using tags
  amazon.aws.backup_selection:
    selection_name: elastic
    backup_plan_name: test-backup-plan
    iam_role_arn: arn:aws:iam::111122223333:role/system-backup
    list_of_tags:
      - condition_type: STRINGEQUALS
        condition_key: Owner
        condition_value: Dev01
"""


RETURN = r"""
backup_selection:
  description: Backup selection details.
  returned: always
  type: complex
  contains:
    backup_plan_id:
      description: ID of the backup plan.
      returned: always
      type: str
      sample: "1111f877-1ecf-4d79-9718-a861cd09df3b"
    creation_date:
      description: Creation date of the backup plan.
      returned: on create/update
      type: str
      sample: '2023-01-24T10:08:03.193000+01:00'
    iam_role_arn:
      description: The ARN of the IAM role that Backup uses to authenticate when backing up the target resource.
      returned: always
      type: str
      sample: "arn:aws:iam::111122223333:role/system-backup"
    selection_id:
      description: Backup selection id.
      returned: always
      type: str
      sample: "1111c217-5d71-4a55-8728-5fc4e63d437b"
    selection_name:
      description: The display name of a resource selection document.
      returned: always
      type: str
      sample: elastic
    conditions:
      description: A list of conditions that defines how resources are assigned to backup plans using tags.
      returned: always
      type: dict
      sample: {
                "string_equals": [],
                "string_like": [],
                "string_not_equals": [],
                "string_not_like": []
            }
      contains:
        string_equals:
          description: Filters the values of tagged resources for only those resources that are tagged with the same value.
          returned: always
          type: list
          sample: []
        string_like:
          description: Filters the values of tagged resources for matching tag values with the use of a wildcard character (*) anywhere in the string.
          returned: always
          type: list
          sample: []
        string_not_equals:
          description: Filters the values of tagged resources for only those resources that are not tagged with the same value.
          returned: always
          type: list
          sample: []
        string_not_like:
          description: Filters the values of tagged resources for non-matching tag values with the use of a wildcard character (*) anywhere in the string.
          returned: always
          type: list
          sample: []
    list_of_tags:
      description: Conditions defined to assign resources to the backup plans using tags.
      returned: always
      type: list
      elements: dict
      sample: [
                {
                    "condition_key": "Environment",
                    "condition_type": "STRINGEQUALS",
                    "condition_value": "Dev"
                }
            ]
      contains:
        condition_key:
          description: The key in a key-value pair.
          returned: always
          type: str
          sample: "Environment"
        condition_type:
          description: An operation applied to a key-value pair used to assign resources to backup plan.
          returned: always
          type: str
          sample: "STRINGEQUALS"
        condition_value:
          description: The value in a key-value pair.
          returned: always
          type: str
          sample: "Dev"
    not_resources:
      description: List of Amazon Resource Names (ARNs) that are excluded from the backup plan.
      returned: always
      type: list
      sample: []
    resources:
      description: List of Amazon Resource Names (ARNs) that are assigned to the backup plan.
      returned: always
      type: list
      sample: []
"""

import json

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.backup import get_plan_details
from ansible_collections.amazon.aws.plugins.module_utils.backup import get_selection_details
from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AWSRetry


def check_for_update(current_selection, backup_selection_data, iam_role_arn):
    update_needed = False
    if current_selection[0].get("IamRoleArn", None) != iam_role_arn:
        update_needed = True

    fields_to_check = ["Resources", "ListOfTags", "NotResources", "Conditions"]
    for field_name in fields_to_check:
        field_value_from_aws = json.dumps(current_selection[0].get(field_name, []), sort_keys=True)
        new_field_value = json.dumps(backup_selection_data.get(field_name, []), sort_keys=True)
        if new_field_value != field_value_from_aws:
            if field_name != "Conditions":
                update_needed = True
            elif not (  # Check that Conditions values are not both empty
                field_value_from_aws
                == '{"StringEquals": [], "StringLike": [], "StringNotEquals": [], "StringNotLike": []}'  # Default AWS Conditions return value
                and new_field_value == "[]"
            ):
                update_needed = True
    return update_needed


def main():
    argument_spec = dict(
        backup_selection_name=dict(type="str", required=True, aliases=["selection_name"]),
        backup_plan_name=dict(type="str", required=True, aliases=["plan_name"]),
        iam_role_arn=dict(type="str"),
        resources=dict(type="list", elements="str"),
        conditions=dict(
            type="dict",
            options=dict(
                string_equals=dict(
                    type="list",
                    default=[],
                    elements="dict",
                    options=dict(
                        condition_key=dict(type="str", no_log=False),
                        condition_value=dict(type="str"),
                    ),
                ),
                string_like=dict(
                    type="list",
                    default=[],
                    elements="dict",
                    options=dict(
                        condition_key=dict(type="str", no_log=False),
                        condition_value=dict(type="str"),
                    ),
                ),
                string_not_equals=dict(
                    type="list",
                    default=[],
                    elements="dict",
                    options=dict(
                        condition_key=dict(type="str", no_log=False),
                        condition_value=dict(type="str"),
                    ),
                ),
                string_not_like=dict(
                    type="list",
                    default=[],
                    elements="dict",
                    options=dict(
                        condition_key=dict(type="str", no_log=False),
                        condition_value=dict(type="str"),
                    ),
                ),
            ),
        ),
        not_resources=dict(type="list", elements="str"),
        list_of_tags=dict(
            type="list",
            elements="dict",
            options=dict(
                condition_type=dict(type="str"),
                condition_key=dict(type="str", no_log=False),
                condition_value=dict(type="str"),
            ),
        ),
        state=dict(default="present", choices=["present", "absent"]),
    )
    required_if = [
        ("state", "present", ["backup_selection_name", "backup_plan_name", "iam_role_arn"]),
        ("state", "absent", ["backup_selection_name", "backup_plan_name"]),
    ]
    module = AnsibleAWSModule(argument_spec=argument_spec, required_if=required_if, supports_check_mode=True)
    state = module.params.get("state")
    backup_selection_name = module.params.get("selection_name")
    backup_plan_name = module.params.get("backup_plan_name")
    iam_role_arn = module.params.get("iam_role_arn")
    resources = module.params.get("resources")
    list_of_tags = module.params.get("list_of_tags")
    not_resources = module.params.get("not_resources")
    conditions = module.params.get("conditions")

    try:
        client = module.client("backup", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    results = {"changed": False, "exists": False, "backup_selection": {}}

    current_selection = get_selection_details(module, client, backup_plan_name, backup_selection_name)
    results["current_selection"] = current_selection

    if state == "present":
        # build data specified by user
        update_needed = False
        backup_selection_data = {"SelectionName": backup_selection_name, "IamRoleArn": iam_role_arn}
        if resources:
            backup_selection_data["Resources"] = resources
        if list_of_tags:
            backup_selection_data["ListOfTags"] = snake_dict_to_camel_dict(list_of_tags, capitalize_first=True)
        if not_resources:
            backup_selection_data["NotResources"] = not_resources
        if conditions:
            backup_selection_data["Conditions"] = snake_dict_to_camel_dict(conditions, capitalize_first=True)

        if current_selection:
            results["exists"] = True
            update_needed = check_for_update(current_selection, backup_selection_data, iam_role_arn)
            if update_needed:
                if module.check_mode:
                    results["changed"] = True
                    module.exit_json(**results, msg="Would have created selection if not in check mode")

                try:
                    client.delete_backup_selection(
                        aws_retry=True,
                        SelectionId=current_selection[0]["SelectionId"],
                        BackupPlanId=current_selection[0]["BackupPlanId"],
                    )
                except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                    module.fail_json_aws(e, msg="Failed to delete selection")
            elif not update_needed:
                results["exists"] = True
        # state is present but backup vault doesnt exist
        if not current_selection or update_needed:
            results["changed"] = True
            results["exists"] = True
            plan = get_plan_details(module, client, backup_plan_name)

            if module.check_mode:
                module.exit_json(**results, msg="Would have created selection if not in check mode")
            try:
                client.create_backup_selection(
                    BackupSelection=backup_selection_data, BackupPlanId=plan[0]["backup_plan_id"]
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to create selection")

        new_selection = get_selection_details(module, client, backup_plan_name, backup_selection_name)
        results["backup_selection"] = camel_dict_to_snake_dict(*new_selection)

    elif state == "absent":
        if current_selection:
            results["changed"] = True
            if module.check_mode:
                module.exit_json(**results, msg="Would have deleted backup selection if not in check mode")
            try:
                client.delete_backup_selection(
                    aws_retry=True,
                    SelectionId=current_selection[0]["SelectionId"],
                    BackupPlanId=current_selection[0]["BackupPlanId"],
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to delete selection")

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2022 Aubin Bikouo (@abikouo)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_multiplemanageddisks

version_added: "1.14.0"

short_description: Manage Multiple Azure Manage Disks

description:
    - Create, update and delete one or more Azure Managed Disk.
    - This module can be used also to attach/detach disks to/from one or more virtual machines.

options:
    state:
        description:
            - Assert the state of the managed disks.
            - Use C(present) to create or update managed disks and/or attach/detach managed disks to a list of
              VMs depending on the value specified in I(managed_by_extended).
            - Use C(absent) to detach/delete managed disks depending on the value specified in I(managed_by_extended).
        default: present
        type: str
        choices:
            - absent
            - present
    managed_disks:
        description:
            - List of managed disks to create, update, or delete.
        type: list
        elements: dict
        suboptions:
            resource_group:
                description:
                    - Name of a resource group where the managed disk exists or will be created.
                required: true
                type: str
            name:
                description:
                    - Name of the managed disk.
                required: true
                type: str
            location:
                description:
                    - Valid Azure location. Defaults to location of the resource group.
                type: str
            storage_account_type:
                description:
                    - Type of storage for the managed disk.
                    - If not specified, the disk is created as C(Standard_LRS).
                    - C(Standard_LRS) is for Standard HDD.
                    - C(StandardSSD_LRS) (added in 2.8) is for Standard SSD.
                    - C(StandardSSD_ZRS) is for Standard SSD Zone-redundant.
                    - C(Premium_LRS) is for Premium SSD.
                    - C(Premium_ZRS) is for Premium SSD Zone-redundant.
                    - C(UltraSSD_LRS) (added in 2.8) is for Ultra SSD, which is only available on select instance types.
                    - See U(https://docs.microsoft.com/en-us/azure/virtual-machines/windows/disks-types) for more information about disk types.
                choices:
                    - Standard_LRS
                    - StandardSSD_LRS
                    - StandardSSD_ZRS
                    - Premium_LRS
                    - Premium_ZRS
                    - UltraSSD_LRS
                type: str
            create_option:
                description:
                    - C(import) from a VHD file in I(source_uri) and C(copy) from previous managed disk I(source_uri).
                type: str
                choices:
                    - empty
                    - import
                    - copy
            storage_account_id:
                description:
                    - The full path to the storage account the image is to be imported from.
                    - Required when I(create_option=import).
                type: str
            source_uri:
                description:
                    - URI to a valid VHD file to be used or the resource ID of the managed disk to copy.
                    - Required when I(create_option=import) or I(create_option=copy).
                aliases:
                    - source_resource_uri
                type: str
            os_type:
                description:
                    - Type of Operating System.
                    - Used when I(create_option=copy) or I(create_option=import) and the source is an OS disk.
                    - If omitted during creation, no value is set.
                    - If omitted during an update, no change is made.
                    - Once set, this value cannot be cleared.
                choices:
                    - linux
                    - windows
                type: str
            disk_size_gb:
                description:
                    - Size in GB of the managed disk to be created.
                    - Required when I(create_option=empty).
                    - If I(create_option=copy) then the value must be greater than or equal to the source's size.
                type: int
            max_shares:
                description:
                    - The maximum number of VMs that can attach to the disk at the same time.
                    - Value greater than one indicates a disk that can be mounted on multiple VMs at the same time.
                type: int
            attach_caching:
                description:
                    - Disk caching policy controlled by VM. Will be used when attached to the VM defined by C(managed_by).
                    - If this option is different from the current caching policy, the managed disk will be deattached
                      and attached with current caching option again.
                choices:
                    - ''
                    - read_only
                    - read_write
                type: str
            zone:
                description:
                    - The Azure managed disk's zone.
                    - Allowed values are C(1), C(2), C(3) and C('').
                choices:
                    - '1'
                    - '2'
                    - '3'
                    - ''
                type: str
            lun:
                description:
                    - The logical unit number for data disk.
                    - This value is used to identify data disks within the VM and therefore must be unique for each data disk attached to a VM.
                type: int
    managed_by_extended:
        description:
            - List of name and resource group of the VMs to managed disks.
            - When I(state=present), the disks will be attached to the list of VMs specified.
            - When I(state=present), use I([]) to detach disks from all the VMs.
            - When I(state=absent) and this parameter is defined, the disks will be detached from the list of VMs.
            - When I(state=absent) and this parameter is not defined, the disks will be deleted.
        type: list
        elements: dict
        suboptions:
            resource_group:
                description:
                    - The resource group of the attache VM.
                type: str
            name:
                description:
                    - The name of the attache VM.
                type: str

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Aubin Bikouo (@abikouo)
'''

EXAMPLES = '''
- name: Create managed operating system disks from page blob and attach them to a list of VMs
  azure_rm_multiplemanageddisks:
    managed_disks:
      - name: mymanageddisk1
        location: eastus2
        resource_group: myResourceGroup
        create_option: import
        source_uri: https://storageaccountname.blob.core.windows.net/containername/blob-name.vhd
        storage_account_id: /subscriptions/<uuid>/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/storageaccountname
        os_type: windows
        storage_account_type: Premium_LRS
      - name: mymanageddisk2
        location: eastus2
        resource_group: myResourceGroup
        create_option: import
        source_uri: https://storageaccountname.blob.core.windows.net/containername/blob-name.vhd
        storage_account_id: /subscriptions/<uuid>/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/storageaccountname
        os_type: windows
        storage_account_type: Premium_LRS
    managed_by_extended:
      - resource_group: myResourceGroupTest
        name: TestVM

- name: Detach disks from the VMs specified in the list
  azure_rm_multiplemanageddisks:
    state: absent
    managed_disks:
      - name: mymanageddisk1
        location: eastus2
        resource_group: myResourceGroup
        create_option: import
        source_uri: https://storageaccountname.blob.core.windows.net/containername/blob-name.vhd
        storage_account_id: /subscriptions/<uuid>/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/storageaccountname
        os_type: windows
        storage_account_type: Premium_LRS
      - name: mymanageddisk2
        location: eastus2
        resource_group: myResourceGroup
        create_option: import
        source_uri: https://storageaccountname.blob.core.windows.net/containername/blob-name.vhd
        storage_account_id: /subscriptions/<uuid>/resourceGroups/myResourceGroup/providers/Microsoft.Storage/storageAccounts/storageaccountname
        os_type: windows
        storage_account_type: Premium_LRS
    managed_by_extended:
      - resource_group: myResourceGroupTest
        name: TestVM1
      - resource_group: myResourceGroupTest
        name: TestVM2

- name: Detach managed disks from all VMs without deletion
  azure_rm_multiplemanageddisks:
    state: present
    managed_disks:
      - name: mymanageddisk1
        location: eastus2
        resource_group: myResourceGroup
      - name: mymanageddisk2
        location: eastus2
        resource_group: myResourceGroup
    managed_by_extended: []

- name: Detach managed disks from all VMs and delete them
  azure_rm_multiplemanageddisks:
    state: absent
    managed_disks:
      - name: mymanageddisk1
        location: eastus2
        resource_group: myResourceGroup
      - name: mymanageddisk2
        location: eastus2
        resource_group: myResourceGroup
'''

RETURN = '''
state:
    description:
        - Current state of the managed disks.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource id.
            type: str
        name:
            description:
                - Name of the managed disk.
            type: str
        location:
            description:
                - Valid Azure location.
            type: str
        storage_account_type:
            description:
                - Type of storage for the managed disk.
                - See U(https://docs.microsoft.com/en-us/azure/virtual-machines/windows/disks-types) for more information about this type.
            type: str
            sample: Standard_LRS
        create_option:
            description:
                - Create option of the disk.
            type: str
            sample: copy
        storage_account_id:
            description:
                - The full path to the storage account the image is to be imported from
            type: str
            sample: /subscriptions/<uuid>/resourceGroups/<resource group name>/providers/Microsoft.Storage/storageAccounts/<storage account name>
        source_uri:
            description:
                - URI to a valid VHD file to be used or the resource ID of the managed disk to copy.
            type: str
        os_type:
            description:
                - Type of Operating System.
            type: str
            sample: linux
        disk_size_gb:
            description:
                - Size in GB of the managed disk to be created.
            type: str
        managed_by:
            description:
                - Name of an existing virtual machine with which the disk is or will be associated, this VM should be in the same resource group.
            type: str
        max_shares:
            description:
                - The maximum number of VMs that can attach to the disk at the same time.
                - Value greater than one indicates a disk that can be mounted on multiple VMs at the same time.
            type: int
            sample: 3
        managed_by_extended:
            description:
                - List ID of an existing virtual machine with which the disk is or will be associated.
            type: list
            sample: ["/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Compute/virtualMachines/testVM"]
        tags:
            description:
                - Tags to assign to the managed disk.
            type: dict
            sample: { "tag": "value" }
'''


from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
try:
    from azure.core.exceptions import ResourceNotFoundError, AzureError
    from azure.mgmt.core.tools import parse_resource_id
    import time
except ImportError:
    # This is handled in azure_rm_common
    pass


# duplicated in azure_rm_manageddisk_facts
def managed_disk_to_dict(managed_disk):
    create_data = managed_disk.creation_data
    return dict(
        id=managed_disk.id,
        name=managed_disk.name,
        location=managed_disk.location,
        tags=managed_disk.tags,
        create_option=create_data.create_option.lower(),
        source_uri=create_data.source_uri or create_data.source_resource_id,
        disk_size_gb=managed_disk.disk_size_gb,
        os_type=managed_disk.os_type.lower() if managed_disk.os_type else None,
        storage_account_type=managed_disk.sku.name if managed_disk.sku else None,
        managed_by=managed_disk.managed_by,
        max_shares=managed_disk.max_shares,
        managed_by_extended=managed_disk.managed_by_extended,
        zone=managed_disk.zones[0] if managed_disk.zones and len(managed_disk.zones) > 0 else ''
    )


class AzureRMMultipleManagedDisk(AzureRMModuleBase):
    """Configuration class for an Azure RM Managed Disk resource"""

    def __init__(self):

        managed_by_extended_spec = dict(
            resource_group=dict(type='str'),
            name=dict(type='str')
        )
        managed_disks_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            storage_account_type=dict(
                type='str',
                choices=['Standard_LRS', 'StandardSSD_LRS', 'StandardSSD_ZRS', 'Premium_LRS', 'Premium_ZRS', 'UltraSSD_LRS']
            ),
            create_option=dict(
                type='str',
                choices=['empty', 'import', 'copy']
            ),
            storage_account_id=dict(
                type='str'
            ),
            source_uri=dict(
                type='str',
                aliases=['source_resource_uri']
            ),
            os_type=dict(
                type='str',
                choices=['linux', 'windows']
            ),
            disk_size_gb=dict(
                type='int'
            ),
            zone=dict(
                type='str',
                choices=['', '1', '2', '3']
            ),
            attach_caching=dict(
                type='str',
                choices=['', 'read_only', 'read_write']
            ),
            lun=dict(
                type='int'
            ),
            max_shares=dict(
                type='int'
            ),
        )

        self.module_arg_spec = dict(
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            managed_disks=dict(
                type='list',
                elements='dict',
                options=managed_disks_spec,
            ),
            managed_by_extended=dict(
                type='list',
                elements='dict',
                options=managed_by_extended_spec,
            ),
        )
        self.results = dict(
            changed=False,
            state=list())

        super(AzureRMMultipleManagedDisk, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_tags=True)

    def validate_disks_parameter(self):
        errors = []
        create_option_reqs = [
            ('import', ['source_uri', 'storage_account_id']),
            ('copy', ['source_uri']),
            ('empty', ['disk_size_gb'])
        ]
        for disk in self.managed_disks:
            create_option = disk.get("create_option")
            for req in create_option_reqs:
                if create_option == req[0] and any((disk.get(opt) is None for opt in req[1])):
                    errors.append("managed disk {0}/{1} has create_option set to {2} but not all required parameters ({3}) are set.".format(
                                  disk.get("resource_group"), disk.get("name"), req[0], ",".join(req[1])))
        if errors:
            self.fail(msg="Some required options are missing from managed disks configuration.", errors=errors)

    def generate_disk_parameters(self, location, tags, zone=None,
                                 storage_account_type=None, disk_size_gb=None, create_option=None,
                                 source_uri=None, storage_account_id=None, os_type=None, max_shares=None, **kwargs):
        disk_params = {}
        creation_data = {}
        disk_params['location'] = location
        disk_params['tags'] = tags
        if zone:
            disk_params['zones'] = [zone]
        if storage_account_type:
            storage = self.multi_disk_models.DiskSku(name=storage_account_type)
            disk_params['sku'] = storage
        disk_params['disk_size_gb'] = disk_size_gb
        creation_data['create_option'] = self.multi_disk_models.DiskCreateOption.empty
        if create_option == 'import':
            creation_data['create_option'] = self.multi_disk_models.DiskCreateOption.import_enum
            creation_data['source_uri'] = source_uri
            creation_data['storage_account_id'] = storage_account_id
        elif create_option == 'copy':
            creation_data['create_option'] = self.multi_disk_models.DiskCreateOption.copy
            creation_data['source_resource_id'] = source_uri
        if os_type:
            disk_params['os_type'] = self.multi_disk_models.OperatingSystemTypes(os_type.capitalize())
        else:
            disk_params['os_type'] = None
        if max_shares:
            disk_params['max_shares'] = max_shares
        disk_params['creation_data'] = creation_data
        return disk_params

    def get_disk_instance(self, managed_disk):
        resource_group = self.get_resource_group(managed_disk.get("resource_group"))
        managed_disk["location"] = managed_disk.get("location") or resource_group.location
        disk_instance = self.get_managed_disk(resource_group=managed_disk.get("resource_group"), name=managed_disk.get("name"))
        if disk_instance is not None:
            for key in ("create_option", 'source_uri', 'disk_size_gb', 'os_type', 'zone'):
                if managed_disk.get(key) is None:
                    managed_disk[key] = disk_instance.get(key)
        parameter = self.generate_disk_parameters(tags=self.tags, **managed_disk)

        return parameter, disk_instance

    def exec_module(self, **kwargs):
        """Main module execution method"""
        self.tags = kwargs.get("tags")

        state = kwargs.get("state")
        self.managed_disks = kwargs.get("managed_disks")
        self.managed_by_extended = kwargs.get("managed_by_extended")

        self.validate_disks_parameter()

        managed_vm_id = []
        if self.managed_by_extended:
            managed_vm_id = [self._get_vm(vm['resource_group'], vm['name']) for vm in self.managed_by_extended]

        if state == "present":
            return self.create_or_attach_disks(managed_vm_id)
        elif state == "absent":
            return self.detach_or_delete_disks(managed_vm_id)

    def compute_disks_result(self, disk_instances):
        result = []
        for params, disk in disk_instances:
            disk_id = parse_resource_id(disk.get("id"))
            result.append(self.get_managed_disk(resource_group=disk_id.get("resource_group"), name=disk_id.get("resource_name")))
        return result

    def create_or_attach_disks(self, managed_vm_id):
        changed, disk_instances, disks_to_create = False, [], []
        for disk in self.managed_disks:
            parameter, disk_instance = self.get_disk_instance(disk)
            # create or update disk
            disk_info_to_compare = dict(zone=disk.get("zone"), max_shares=disk.get("max_shares"), found_disk=disk_instance, new_disk=parameter)
            if disk_instance is None or self.is_different(**disk_info_to_compare):
                disks_to_create.append((disk, parameter))
            else:
                disk_instances.append((disk, disk_instance))

        if len(disks_to_create) > 0:
            changed = True
            result = self.create_or_update_disks(disks_to_create)
            disk_instances += result

        if self.managed_by_extended is not None and len(self.managed_by_extended) > 0:
            # Attach the disk to multiple VM
            attach_config = []
            for vm in managed_vm_id:
                time.sleep(5)
                disks = [(d, i) for d, i in disk_instances if not self._is_disk_attached_to_vm(vm.id, i)]
                if len(disks) > 0:
                    attach_config.append(self.create_attachment_configuration(vm, disks))

            if len(attach_config) > 0:
                changed = True
                self.update_virtual_machines(attach_config)

        elif self.managed_by_extended == []:
            # Detach disks from all VMs attaching them
            changed = self.detach_disks_from_all_vms(disk_instances) or changed

        return dict(changed=changed, state=self.compute_disks_result(disk_instances))

    def detach_or_delete_disks(self, managed_vm_id):
        changed, disk_instances = False, []
        for disk in self.managed_disks:
            params, disk_instance = self.get_disk_instance(disk)
            if disk_instance is not None:
                disk_instances.append((disk, disk_instance))

        result = []
        if self.managed_by_extended is not None and len(self.managed_by_extended) > 0:
            # Detach the disk from list of VMs
            disks_names = [d.get("name").lower() for p, d in disk_instances]
            attach_config = []
            for vm in managed_vm_id:
                disks = [d for p, d in disk_instances if self._is_disk_attached_to_vm(vm.id, d)]
                if len(disks) > 0:
                    attach_config.append(self.create_detachment_configuration(vm, disks_names))

            if len(attach_config) > 0:
                changed = True
                self.update_virtual_machines(attach_config)
            result = self.compute_disks_result(disk_instances)

        elif self.managed_by_extended is None:
            # Detach disks from all VMs attaching them
            changed = self.detach_disks_from_all_vms(disk_instances)

            # Delete existing disks
            if len(disk_instances) > 0:
                disks_ids = [disk.get("id") for param, disk in disk_instances]
                changed = True
                self.delete_disks(disks_ids)

        return dict(changed=changed, state=result)

    def detach_disks_from_all_vms(self, disk_instances):
        changed = False
        # Detach disk to all VMs attaching it
        unique_vm_id = []
        for param, disk_instance in disk_instances:
            managed_by_vm = disk_instance.get("managed_by")
            managed_by_extended_vms = disk_instance.get("managed_by_extended") or []
            if managed_by_vm is not None and managed_by_vm not in unique_vm_id:
                unique_vm_id.append(managed_by_vm)
            for vm_id in managed_by_extended_vms:
                if vm_id not in unique_vm_id:
                    unique_vm_id.append(vm_id)
        if unique_vm_id:
            disks_names = [instance.get("name").lower() for d, instance in disk_instances]
            changed = True
            attach_config = []
            for vm_id in unique_vm_id:
                vm_name_id = parse_resource_id(vm_id)
                vm_instance = self._get_vm(vm_name_id['resource_group'], vm_name_id['resource_name'])
                attach_config.append(self.create_detachment_configuration(vm_instance, disks_names))

            if len(attach_config) > 0:
                changed = True
                self.update_virtual_machines(attach_config)
        return changed

    def _is_disk_attached_to_vm(self, vm_id, item):
        managed_by = item['managed_by']
        managed_by_extended = item['managed_by_extended']
        if managed_by is not None and vm_id == managed_by:
            return True
        if managed_by_extended is not None and vm_id in managed_by_extended:
            return True
        return False

    def create_attachment_configuration(self, vm, disks):
        vm_id = parse_resource_id(vm.id)

        # attach all disks to the virtual machine
        for managed_disk, disk_instance in disks:
            lun = managed_disk.get("lun")
            if lun is None:
                luns = ([d.lun for d in vm.storage_profile.data_disks] if vm.storage_profile.data_disks else [])
                lun = 0
                while True:
                    if lun not in luns:
                        break
                    lun = lun + 1
                for item in vm.storage_profile.data_disks:
                    if item.name == managed_disk.get("name"):
                        lun = item.lun

            # prepare the data disk
            params = self.multi_disk_models.ManagedDiskParameters(id=disk_instance.get('id'), storage_account_type=disk_instance.get('storage_account_type'))
            attach_caching = managed_disk.get("attach_caching")
            caching_options = self.multi_disk_models.CachingTypes[attach_caching] if attach_caching and attach_caching != '' else None

            # pylint: disable=missing-kwoa
            data_disk = self.multi_disk_models.DataDisk(lun=lun,
                                                        create_option=self.compute_models.DiskCreateOptionTypes.attach,
                                                        managed_disk=params,
                                                        caching=caching_options)
            vm.storage_profile.data_disks.append(data_disk)
        return vm_id["resource_group"], vm_id["resource_name"], vm

    def create_detachment_configuration(self, vm_instance, disks_names):
        vm_data = parse_resource_id(vm_instance.id)
        leftovers = [d for d in vm_instance.storage_profile.data_disks if d.name.lower() not in disks_names]
        if len(vm_instance.storage_profile.data_disks) == len(leftovers):
            self.fail("None of the following disks '{0}' are attached to the VM '{1}/{2}'.".format(
                disks_names, vm_data["resource_group"], vm_data["resource_name"]
            ))
        vm_instance.storage_profile.data_disks = leftovers
        return vm_data["resource_group"], vm_data["resource_name"], vm_instance

    def _get_vm(self, resource_group, name):
        try:
            return self.compute_client.virtual_machines.get(resource_group, name, expand='instanceview')
        except Exception as exc:
            self.fail("Error getting virtual machine {0}/{1} - {2}".format(resource_group, name, str(exc)))

    def create_or_update_disks(self, disks_to_create):
        pollers = []
        for disk_info, disk in disks_to_create:
            resource_group = disk_info.get("resource_group")
            name = disk_info.get("name")
            try:
                poller = self.multi_disk_client.disks.begin_create_or_update(resource_group, name, disk)
                pollers.append(poller)
            except Exception as e:
                self.fail("Error creating the managed disk {0}/{1}: {2}".format(resource_group, name, str(e)))
        disks_instances = self.get_multiple_pollers_results(pollers)
        result = []
        for i, instance in enumerate(disks_instances):
            result.append((disks_to_create[i][0], managed_disk_to_dict(instance)))
        return result

    # This method accounts for the difference in structure between the
    # Azure retrieved disk and the parameters for the new disk to be created.
    def is_different(self, zone, max_shares, found_disk, new_disk):
        resp = False
        if new_disk.get('disk_size_gb'):
            if not found_disk['disk_size_gb'] == new_disk['disk_size_gb']:
                resp = True
        if new_disk.get('os_type'):
            if found_disk['os_type'] is None or not self.multi_disk_models.OperatingSystemTypes(found_disk['os_type'].capitalize()) == new_disk['os_type']:
                resp = True
        if new_disk.get('sku'):
            if not found_disk['storage_account_type'] == new_disk['sku'].name:
                resp = True
        # Check how to implement tags
        if new_disk.get('tags') is not None:
            if not found_disk['tags'] == new_disk['tags']:
                resp = True
        if zone is not None:
            if not found_disk['zone'] == zone:
                resp = True
        if max_shares is not None:
            if not found_disk['max_shares'] == max_shares:
                resp = True
        return resp

    def delete_disks(self, ids):
        pollers = []
        for disk_id in ids:
            try:
                disk = parse_resource_id(disk_id)
                resource_group, name = disk.get("resource_group"), disk.get("resource_name")
                poller = self.multi_disk_client.disks.begin_delete(resource_group, name)
                pollers.append(poller)
            except Exception as e:
                self.fail("Error deleting the managed disk {0}/{1}: {2}".format(resource_group, name, str(e)))
        return self.get_multiple_pollers_results(pollers)

    def update_virtual_machines(self, config):
        pollers = []
        for resource_group, name, params in config:
            try:
                poller = self.compute_client.virtual_machines.begin_create_or_update(resource_group, name, params)
                pollers.append(poller)
            except AzureError as exc:
                self.fail("Error updating virtual machine (attaching/detaching disks) {0}/{1} - {2}".format(resource_group, name, exc.message))
        return self.get_multiple_pollers_results(pollers)

    def get_managed_disk(self, resource_group, name):
        try:
            resp = self.multi_disk_client.disks.get(resource_group, name)
            return managed_disk_to_dict(resp)
        except ResourceNotFoundError:
            self.log("Did not find managed disk {0}/{1}".format(resource_group, name))


def main():
    """Main execution"""
    AzureRMMultipleManagedDisk()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: device_reboot_apreboot_info
short_description: Information module for Device Reboot Apreboot Info
description:
- This module represents an alias of the module device_reboot_apreboot_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  parentTaskId:
    description:
    - ParentTaskId query parameter. Task id of ap reboot request.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless GetAccessPointRebootTaskResultV1
  description: Complete reference of the GetAccessPointRebootTaskResultV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-access-point-reboot-task-result
notes:
  - SDK Method used are
    wireless.Wireless.get_access_point_reboot_task_result_v1,

  - Paths used are
    get /dna/intent/api/v1/device-reboot/apreboot/status,
  - It should be noted that this module is an alias of device_reboot_apreboot_v1_info

"""

EXAMPLES = r"""
- name: Get all Device Reboot Apreboot Info
  cisco.dnac.device_reboot_apreboot_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    parentTaskId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "wlcIP": "string",
        "apList": [
          {
            "apName": "string",
            "rebootStatus": "string",
            "failureReason": {}
          }
        ]
      }
    ]
"""



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The replace_keys filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    name: replace_keys
    author: Sagar Paul (@KB-perByte)
    version_added: "2.5.0"
    short_description: Replaces specific keys with their after value from a data recursively.
    description:
        - This plugin replaces specific keys with their after value from a data recursively.
        - Matching parameter defaults to equals unless C(matching_parameter) is explicitly mentioned.
        - Using the parameters below- C(data|ansible.utils.replace_keys(target([....])))
    options:
      data:
        description:
        - This option represents a list of dictionaries or a dictionary with any level of nesting data.
        - For example C(config_data|ansible.utils.replace_keys(target([....]))), in this case C(config_data) represents this option.
        type: raw
        required: True
      target:
        description: Specify the target keys to replace in list of dictionaries format containing
          before and after key value.
        type: list
        elements: dict
        required: True
        suboptions:
          before:
            description: before attribute key [to change]
            type: str
          after:
            description: after attribute key [change to]
            type: str
      matching_parameter:
        description: Specify the matching configuration of target keys and data attributes.
        type: str
        choices: ["starts_with","ends_with","regex"]
"""

EXAMPLES = r"""
# example.yaml
# interfaces:
#   - interface_name: eth0
#     enabled: true
#     duplex: auto
#     speed: auto
#   - interface_name: eth1
#     description: Configured by Ansible - Interface 1
#     mtu: 1500
#     speed: auto
#     duplex: auto
#     is_enabled: true
#     vifs:
#     - vlan_id: 100
#       description: Eth1 - VIF 100
#       mtu: 400
#       is_enabled: true
#     - vlan_id: 101
#       description: Eth1 - VIF 101
#       is_enabled: true
#   - interface_name: eth2
#     description: Configured by Ansible - Interface 2 (ADMIN DOWN)
#     mtu: 600
#     is_enabled: false

# Playbook
- name: replace keys with specified keys dict/list to dict
  ansible.builtin.set_fact:
    data: '{{ interfaces }}'
- debug:
    msg: >-
      {{ data|ansible.utils.replace_keys(target=[{'before':'interface_name',
      'after':'name'}, {'before':'is_enabled', 'after':'enabled'}]) }}

# Output
# TASK [replace keys with specified keys dict/list to dict] *************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": [
#             {
#                 "duplex": "auto",
#                 "enabled": true,
#                 "interface_name": "eth0",
#                 "speed": "auto"
#             },
#             {
#                 "description": "Configured by Ansible - Interface 1",
#                 "duplex": "auto",
#                 "interface_name": "eth1",
#                 "is_enabled": true,
#                 "mtu": 1500,
#                 "speed": "auto",
#                 "vifs": [
#                     {
#                         "description": "Eth1 - VIF 100",
#                         "is_enabled": true,
#                         "mtu": 400,
#                         "vlan_id": 100
#                     },
#                     {
#                         "description": "Eth1 - VIF 101",
#                         "is_enabled": true,
#                         "vlan_id": 101
#                     }
#                 ]
#             },
#             {
#                 "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#                 "interface_name": "eth2",
#                 "is_enabled": false,
#                 "mtu": 600
#             }
#         ]
#     },
#     "changed": false
# }

# TASK [debug] **********************************************************************
# ok: [localhost] => {
#     "msg": [
#         {
#             "duplex": "auto",
#             "enabled": true,
#             "name": "eth0",
#             "speed": "auto"
#         },
#         {
#             "description": "Configured by Ansible - Interface 1",
#             "duplex": "auto",
#             "enabled": true,
#             "mtu": 1500,
#             "name": "eth1",
#             "speed": "auto",
#             "vifs": [
#                 {
#                     "description": "Eth1 - VIF 100",
#                     "enabled": true,
#                     "mtu": 400,
#                     "vlan_id": 100
#                 },
#                 {
#                     "description": "Eth1 - VIF 101",
#                     "enabled": true,
#                     "vlan_id": 101
#                 }
#             ]
#         },
#         {
#             "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#             "enabled": false,
#             "mtu": 600,
#             "name": "eth2"
#         }
#     ]
# }

# example.yaml
# interfaces:
#   - interface_name: eth0
#     enabled: true
#     duplex: auto
#     speed: auto
#   - interface_name: eth1
#     description: Configured by Ansible - Interface 1
#     mtu: 1500
#     speed: auto
#     duplex: auto
#     is_enabled: true
#     vifs:
#     - vlan_id: 100
#       description: Eth1 - VIF 100
#       mtu: 400
#       is_enabled: true
#     - vlan_id: 101
#       description: Eth1 - VIF 101
#       is_enabled: true
#   - interface_name: eth2
#     description: Configured by Ansible - Interface 2 (ADMIN DOWN)
#     mtu: 600
#     is_enabled: false

# Playbook
- name: replace keys with specified keys dict/list to dict
  ansible.builtin.set_fact:
    data: '{{ interfaces }}'
- debug:
    msg: >-
      {{ data|ansible.utils.replace_keys(target=[{'before':'name',
      'after':'name'}, {'before':'enabled', 'after':'enabled'}],
      matching_parameter= 'ends_with') }}

# Output
# TASK [replace keys with specified keys dict/list to dict] *********************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": [
#             {
#                 "duplex": "auto",
#                 "enabled": true,
#                 "interface_name": "eth0",
#                 "speed": "auto"
#             },
#             {
#                 "description": "Configured by Ansible - Interface 1",
#                 "duplex": "auto",
#                 "interface_name": "eth1",
#                 "is_enabled": true,
#                 "mtu": 1500,
#                 "speed": "auto",
#                 "vifs": [
#                     {
#                         "description": "Eth1 - VIF 100",
#                         "is_enabled": true,
#                         "mtu": 400,
#                         "vlan_id": 100
#                     },
#                     {
#                         "description": "Eth1 - VIF 101",
#                         "is_enabled": true,
#                         "vlan_id": 101
#                     }
#                 ]
#             },
#             {
#                 "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#                 "interface_name": "eth2",
#                 "is_enabled": false,
#                 "mtu": 600
#             }
#         ]
#     },
#     "changed": false
# }

# TASK [debug] ***************************************************************************
# ok: [localhost] => {
#     "msg": [
#         {
#             "duplex": "auto",
#             "enabled": true,
#             "name": "eth0",
#             "speed": "auto"
#         },
#         {
#             "description": "Configured by Ansible - Interface 1",
#             "duplex": "auto",
#             "enabled": true,
#             "mtu": 1500,
#             "name": "eth1",
#             "speed": "auto",
#             "vifs": [
#                 {
#                     "description": "Eth1 - VIF 100",
#                     "enabled": true,
#                     "mtu": 400,
#                     "vlan_id": 100
#                 },
#                 {
#                     "description": "Eth1 - VIF 101",
#                     "enabled": true,
#                     "vlan_id": 101
#                 }
#             ]
#         },
#         {
#             "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#             "enabled": false,
#             "mtu": 600,
#             "name": "eth2"
#         }
#     ]
# }
"""

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.replace_keys import replace_keys


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _replace_keys(*args, **kwargs):
    """replaces specific keys with their after value from a data recursively"""

    keys = ["data", "target", "matching_parameter"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="replace_keys")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return replace_keys(**updated_data)


class FilterModule(object):
    """replace_keys"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"replace_keys": _replace_keys}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: network_device
short_description: Resource module for Network Device
description:
- This module represents an alias of the module network_device_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  cleanConfig:
    description: CleanConfig query parameter. Selecting the clean up configuration option
      will attempt to remove device settings that were configured during the addition
      of the device to the inventory and site assignment. Please note that this operation
      is different from deprovisioning. It does not remove configurations that were
      pushed during device provisioning.
    type: bool
    version_added: 4.0.0
  cliTransport:
    description: CLI transport. Supported values telnet, ssh. Required if type is NETWORK_DEVICE.
    type: str
  computeDevice:
    description: Compute Device or not. Options are true / false.
    type: bool
  enablePassword:
    description: CLI enable password of the device. Required if device is configured
      to use enable password.
    type: str
  extendedDiscoveryInfo:
    description: This field holds that info as whether to add device with canned data
      or not. Supported values DISCOVER_WITH_CANNED_DATA.
    type: str
  httpPassword:
    description: HTTP password of the device / API key for Meraki Dashboard. Required
      if type is MERAKI_DASHBOARD or COMPUTE_DEVICE.
    type: str
  httpPort:
    description: HTTP port of the device. Required if type is COMPUTE_DEVICE.
    type: str
  httpSecure:
    description: Flag to select HTTP / HTTPS protocol. Options are true / false. True
      for HTTPS and false for HTTP. Default is true.
    type: bool
  httpUserName:
    description: HTTP Username of the device. Required if type is COMPUTE_DEVICE.
    type: str
  id:
    description: Id path parameter. Device ID.
    type: str
  ipAddress:
    description: IP Address of the device. Required if type is NETWORK_DEVICE, COMPUTE_DEVICE
      or THIRD_PARTY_DEVICE.
    elements: str
    type: list
  merakiOrgId:
    description: Selected Meraki organization for which the devices needs to be imported.
      Required if type is MERAKI_DASHBOARD.
    elements: str
    type: list
  netconfPort:
    description: Netconf Port of the device. CliTransport must be 'ssh' if netconf is
      provided.
    type: str
  password:
    description: CLI Password of the device. Required if type is NETWORK_DEVICE.
    type: str
  serialNumber:
    description: Serial Number of the Device. Required if extendedDiscoveryInfo is 'DISCOVER_WITH_CANNED_DATA'.
    type: str
  snmpAuthPassphrase:
    description: SNMPv3 auth passphrase of the device. Required if snmpMode is authNoPriv
      or authPriv.
    type: str
  snmpAuthProtocol:
    description: SNMPv3 auth protocol. Supported values sha, md5. Required if snmpMode
      is authNoPriv or authPriv.
    type: str
  snmpMode:
    description: SNMPv3 mode. Supported values noAuthnoPriv, authNoPriv, authPriv. Required
      if snmpVersion is v3.
    type: str
  snmpPrivPassphrase:
    description: SNMPv3 priv passphrase. Required if snmpMode is authPriv.
    type: str
  snmpPrivProtocol:
    description: SNMPv3 priv protocol. Supported values AES128. Required if snmpMode
      is authPriv.
    type: str
  snmpROCommunity:
    description: SNMP Read Community of the device. If snmpVersion is v2, at least one
      of snmpROCommunity and snmpRWCommunity is required.
    type: str
  snmpRWCommunity:
    description: SNMP Write Community of the device. If snmpVersion is v2, at least
      one of snmpROCommunity and snmpRWCommunity is required.
    type: str
  snmpRetry:
    description: SNMP retry count. Max value supported is 3. Default is Global SNMP
      retry (if exists) or 3.
    type: int
  snmpTimeout:
    description: SNMP timeout in seconds. Max value supported is 300. Default is Global
      SNMP timeout (if exists) or 5.
    type: int
  snmpUserName:
    description: SNMPV3 user name of the device. Required if snmpVersion is v3.
    type: str
  snmpVersion:
    description: SNMP version. Values supported v2, v3. Required if type is NETWORK_DEVICE,
      COMPUTE_DEVICE or THIRD_PARTY_DEVICE.
    type: str
  type:
    description: Type of device being added. Default is NETWORK_DEVICE.
    type: str
  updateMgmtIPaddressList:
    description: Network Device's updateMgmtIPaddressList.
    elements: dict
    suboptions:
      existMgmtIpAddress:
        description: ExistMgmtIpAddress IP Address of the device.
        type: str
      newMgmtIpAddress:
        description: New IP Address to be Updated.
        type: str
    type: list
  userName:
    description: CLI user name of the device. Required if type is NETWORK_DEVICE.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices AddDeviceKnowYourNetworkV1
  description: Complete reference of the AddDeviceKnowYourNetworkV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-device-know-your-network
- name: Cisco DNA Center documentation for Devices DeleteDeviceByIdV1
  description: Complete reference of the DeleteDeviceByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-device-by-id
- name: Cisco DNA Center documentation for Devices UpdateDeviceDetailsV1
  description: Complete reference of the UpdateDeviceDetailsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-device-details
notes:
  - SDK Method used are
    devices.Devices.add_device,
    devices.Devices.delete_device_by_id_v1,
    devices.Devices.sync_devices,

  - Paths used are
    post /dna/intent/api/v1/network-device,
    delete /dna/intent/api/v1/network-device/{id},
    put /dna/intent/api/v1/network-device,
  - It should be noted that this module is an alias of network_device_v1

  - Removed 'managementIpAddress' options in v4.3.0.
"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.network_device:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    cliTransport: string
    computeDevice: true
    enablePassword: string
    extendedDiscoveryInfo: string
    httpPassword: string
    httpPort: string
    httpSecure: true
    httpUserName: string
    ipAddress:
    - string
    merakiOrgId:
    - string
    netconfPort: string
    password: string
    serialNumber: string
    snmpAuthPassphrase: string
    snmpAuthProtocol: string
    snmpMode: string
    snmpPrivPassphrase: string
    snmpPrivProtocol: string
    snmpROCommunity: string
    snmpRWCommunity: string
    snmpRetry: 0
    snmpTimeout: 0
    snmpUserName: string
    snmpVersion: string
    type: string
    userName: string

- name: Update all
  cisco.dnac.network_device:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    cliTransport: string
    computeDevice: true
    enablePassword: string
    extendedDiscoveryInfo: string
    httpPassword: string
    httpPort: string
    httpSecure: true
    httpUserName: string
    ipAddress:
    - string
    merakiOrgId:
    - string
    netconfPort: string
    password: string
    serialNumber: string
    snmpAuthPassphrase: string
    snmpAuthProtocol: string
    snmpMode: string
    snmpPrivPassphrase: string
    snmpPrivProtocol: string
    snmpROCommunity: string
    snmpRWCommunity: string
    snmpRetry: 0
    snmpTimeout: 0
    snmpUserName: string
    snmpVersion: string
    type: string
    updateMgmtIPaddressList:
    - existMgmtIpAddress: string
      newMgmtIpAddress: string
    userName: string

- name: Delete by id
  cisco.dnac.network_device:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    cleanConfig: true
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: dna_event_snmp_config_info
short_description: Information module for Dna Event Snmp Config Info
description:
- This module represents an alias of the module dna_event_snmp_config_v1_info
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  configId:
    description:
    - ConfigId query parameter. List of SNMP configurations.
    type: str
  offset:
    description:
    - Offset query parameter. The number of SNMP configuration's to offset in the resultset whose default value 0.
    type: float
  limit:
    description:
    - Limit query parameter. The number of SNMP configuration's to limit in the resultset whose default value 10.
    type: float
  sortBy:
    description:
    - SortBy query parameter. SortBy field name.
    type: str
  order:
    description:
    - Order query parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Event Management GetSNMPDestinationV1
  description: Complete reference of the GetSNMPDestinationV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-snmp-destination
notes:
  - SDK Method used are
    event_management.EventManagement.get_snmp_destination_v1,

  - Paths used are
    get /dna/intent/api/v1/dna-event/snmp-config,
  - It should be noted that this module is an alias of dna_event_snmp_config_v1_info

"""

EXAMPLES = r"""
- name: Get all Dna Event Snmp Config Info
  cisco.dnac.dna_event_snmp_config_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    configId: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "version": "string",
        "tenantId": "string",
        "configId": "string",
        "name": "string",
        "description": "string",
        "ipAddress": "string",
        "port": 0,
        "snmpVersion": "string",
        "community": "string",
        "userName": "string",
        "snmpMode": "string",
        "snmpAuthType": "string",
        "authPassword": "string",
        "snmpPrivacyType": "string",
        "privacyPassword": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sda_anycast_gateways_info
short_description: Information module for Sda Anycast Gateways Info
description:
- This module represents an alias of the module sda_anycast_gateways_v1_info
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id query parameter. ID of the anycast gateway.
    type: str
  fabricId:
    description:
    - FabricId query parameter. ID of the fabric the anycast gateway is assigned to.
    type: str
  virtualNetworkName:
    description:
    - VirtualNetworkName query parameter. Name of the virtual network associated with the anycast gateways.
    type: str
  ipPoolName:
    description:
    - IpPoolName query parameter. Name of the IP pool associated with the anycast gateways.
    type: str
  vlanName:
    description:
    - VlanName query parameter. VLAN name of the anycast gateways.
    type: str
  vlanId:
    description:
    - >
      VlanId query parameter. VLAN ID of the anycast gateways. The allowed range for vlanId is 2-4093 except for
      reserved VLANs 1002-1005, 2046, and 4094.
    type: float
  offset:
    description:
    - Offset query parameter. Starting record for pagination.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA GetAnycastGatewaysV1
  description: Complete reference of the GetAnycastGatewaysV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-anycast-gateways
notes:
  - SDK Method used are
    sda.Sda.get_anycast_gateways_v1,

  - Paths used are
    get /dna/intent/api/v1/sda/anycastGateways,
  - It should be noted that this module is an alias of sda_anycast_gateways_v1_info

"""

EXAMPLES = r"""
- name: Get all Sda Anycast Gateways Info
  cisco.dnac.sda_anycast_gateways_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    fabricId: string
    virtualNetworkName: string
    ipPoolName: string
    vlanName: string
    vlanId: 0
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "fabricId": "string",
          "virtualNetworkName": "string",
          "ipPoolName": "string",
          "tcpMssAdjustment": 0,
          "vlanName": "string",
          "vlanId": 0,
          "trafficType": "string",
          "poolType": "string",
          "securityGroupName": "string",
          "isCriticalPool": true,
          "isLayer2FloodingEnabled": true,
          "isWirelessPool": true,
          "isIpDirectedBroadcast": true,
          "isIntraSubnetRoutingEnabled": true,
          "isMultipleIpToMacAddresses": true,
          "isSupplicantBasedExtendedNodeOnboarding": true,
          "isGroupBasedPolicyEnforcementEnabled": true
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: pnp_virtual_account_devices_sync
short_description: Resource module for Pnp Virtual Account Devices Sync
description:
- This module represents an alias of the module pnp_virtual_account_devices_sync_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  autoSyncPeriod:
    description: Pnp Virtual Account Devices Sync's autoSyncPeriod.
    type: int
  ccoUser:
    description: Pnp Virtual Account Devices Sync's ccoUser.
    type: str
  expiry:
    description: Pnp Virtual Account Devices Sync's expiry.
    type: int
  lastSync:
    description: Pnp Virtual Account Devices Sync's lastSync.
    type: int
  profile:
    description: Pnp Virtual Account Devices Sync's profile.
    suboptions:
      addressFqdn:
        description: Pnp Virtual Account Devices Sync's addressFqdn.
        type: str
      addressIpV4:
        description: Pnp Virtual Account Devices Sync's addressIpV4.
        type: str
      cert:
        description: Pnp Virtual Account Devices Sync's cert.
        type: str
      makeDefault:
        description: MakeDefault flag.
        type: bool
      name:
        description: Pnp Virtual Account Devices Sync's name.
        type: str
      port:
        description: Pnp Virtual Account Devices Sync's port.
        type: int
      profileId:
        description: Pnp Virtual Account Devices Sync's profileId.
        type: str
      proxy:
        description: Proxy flag.
        type: bool
    type: dict
  smartAccountId:
    description: Pnp Virtual Account Devices Sync's smartAccountId.
    type: str
  syncResult:
    description: Pnp Virtual Account Devices Sync's syncResult.
    suboptions:
      syncList:
        description: Pnp Virtual Account Devices Sync's syncList.
        elements: dict
        suboptions:
          deviceSnList:
            description: Pnp Virtual Account Devices Sync's deviceSnList.
            elements: str
            type: list
          syncType:
            description: Pnp Virtual Account Devices Sync's syncType.
            type: str
        type: list
      syncMsg:
        description: Pnp Virtual Account Devices Sync's syncMsg.
        type: str
    type: dict
  syncResultStr:
    description: Pnp Virtual Account Devices Sync's syncResultStr.
    type: str
  syncStartTime:
    description: Pnp Virtual Account Devices Sync's syncStartTime.
    type: int
  syncStatus:
    description: Pnp Virtual Account Devices Sync's syncStatus.
    type: str
  tenantId:
    description: Pnp Virtual Account Devices Sync's tenantId.
    type: str
  token:
    description: Pnp Virtual Account Devices Sync's token.
    type: str
  virtualAccountId:
    description: Pnp Virtual Account Devices Sync's virtualAccountId.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) SyncVirtualAccountDevicesV1
  description: Complete reference of the SyncVirtualAccountDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!sync-virtual-account-devices
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.sync_virtual_account_devices_v1,

  - Paths used are
    post /dna/intent/api/v1/onboarding/pnp-device/vacct-sync,
  - It should be noted that this module is an alias of pnp_virtual_account_devices_sync_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.pnp_virtual_account_devices_sync:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    autoSyncPeriod: 0
    ccoUser: string
    expiry: 0
    lastSync: 0
    profile:
      addressFqdn: string
      addressIpV4: string
      cert: string
      makeDefault: true
      name: string
      port: 0
      profileId: string
      proxy: true
    smartAccountId: string
    syncResult:
      syncList:
      - deviceSnList:
        - string
        syncType: string
      syncMsg: string
    syncResultStr: string
    syncStartTime: 0
    syncStatus: string
    tenantId: string
    token: string
    virtualAccountId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "virtualAccountId": "string",
      "autoSyncPeriod": 0,
      "syncResultStr": "string",
      "profile": {
        "proxy": true,
        "makeDefault": true,
        "port": 0,
        "profileId": "string",
        "name": "string",
        "addressIpV4": "string",
        "cert": "string",
        "addressFqdn": "string"
      },
      "ccoUser": "string",
      "syncResult": {
        "syncList": [
          {
            "syncType": "string",
            "deviceSnList": [
              "string"
            ]
          }
        ],
        "syncMsg": "string"
      },
      "token": "string",
      "syncStartTime": 0,
      "lastSync": 0,
      "tenantId": "string",
      "smartAccountId": "string",
      "expiry": 0,
      "syncStatus": "string"
    }
"""
